/******************************************************************************
 * DISCLAIMER

 * This software is supplied by Renesas Electronics Corporation and is only
 * intended for use with Renesas products. No other uses are authorized.

 * This software is owned by Renesas Electronics Corporation and is protected under
 * all applicable laws, including copyright laws.

 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES
 * REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY,
 * INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE AND NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY
 * DISCLAIMED.

 * TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
 * ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
 * FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
 * FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS
 * AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

 * Renesas reserves the right, without notice, to make changes to this
 * software and to discontinue the availability of this software.
 * By using this software, you agree to the additional terms and
 * conditions found by accessing the following link:
 * http://www.renesas.com/disclaimer
 ******************************************************************************/
/* Copyright (C) 2011 Renesas Electronics Corporation. All rights reserved.  */
/******************************************************************************
 * File Name    : FreeRTOS_hooks.c
 * Version      : 1.00
 * Device(s)    : Renesas
 * Tool-Chain   : GNU
 * OS           : FreeRTOS
 * H/W Platform : Renesas
 * Description  : Hooks for FreeRTOS
 ******************************************************************************
 * History : DD.MM.YYYY Version Description
 *         : dd.mm.yyyy 1.00    First Release
 ******************************************************************************/

/******************************************************************************
 Includes   <System Includes> , "Project Includes"
 ******************************************************************************/
#include <stdio.h>
#include "FreeRTOS.h"
#include "task.h"
#include "trace.h"

/******************************************************************************
 Defines
 ******************************************************************************/

/******************************************************************************
 Constant Macros
 ******************************************************************************/

/******************************************************************************
 Function Macros
 ******************************************************************************/

/******************************************************************************
 Typedef definitions
 ******************************************************************************/

/******************************************************************************
 Enumerated Types
 ******************************************************************************/

/******************************************************************************
 Macro definitions
 ******************************************************************************/

/******************************************************************************
 Constant Data
 ******************************************************************************/

/******************************************************************************
 Imported global variables and functions (from other files)
 ******************************************************************************/

/* Prototypes for the standard FreeRTOS callback/hook functions implemented
 within this file. */
extern void vApplicationMallocFailedHook (void);
extern void vApplicationIdleHook (void);
extern void vApplicationStackOverflowHook (xTaskHandle pxTask, signed char *pcTaskName);
extern void vApplicationTickHook (void);
extern void abort (void);

/******************************************************************************
 Exported global variables and functions (to be accessed by other files)
 ******************************************************************************/

/******************************************************************************
 * Function Name: vApplicationMallocFailedHook
 * Description  : The FreeRTOS malloc failed hook
 * Arguments    : none
 * Return Value : none
 ******************************************************************************/
void vApplicationMallocFailedHook (void)
{
    //TODO: fix comment
    /* Called if a call to pvPortMalloc() fails because there is insufficient
       free memory available in the FreeRTOS heap.  pvPortMalloc() is called
       internally by FreeRTOS API functions that create tasks, queues, software
       timers, and semaphores.  The size of the FreeRTOS heap is set by the
       configTOTAL_HEAP_SIZE configuration constant in FreeRTOSConfig.h. */
    taskDISABLE_INTERRUPTS();
    for (;;)
    {
        ;
    };
}
/******************************************************************************
 End of function  vApplicationMallocFailedHook
 ******************************************************************************/

/******************************************************************************
 * Function Name: vApplicationStackOverflowHook
 * Description  : The FreeRTOS task stack overflow hook
 * Arguments    : IN  pxTask - The task handle
 : IN  pcTaskName - The task name
 * Return Value : none
 ******************************************************************************/
void vApplicationStackOverflowHook (xTaskHandle pxTask, signed char *pcTaskName)
{
    (void) pcTaskName;
    (void) pxTask;

    /* Run time stack overflow checking is performed if
     configCHECK_FOR_STACK_OVERFLOW is defined to 1 or 2.  This hook
     function is called if a stack overflow is detected. */
    taskDISABLE_INTERRUPTS();
    for (;;)
    {
        ;
    }
}
/******************************************************************************
 End of function  vApplicationStackOverflowHook
 ******************************************************************************/

/******************************************************************************
 * Function Name: vApplicationIdleHook
 * Description  : The FreeRTOS idle application hook
 * Arguments    : none
 * Return Value : none
 ******************************************************************************/
void vApplicationIdleHook (void)
{
    volatile size_t xFreeHeapSpace;

    /* This is just a trivial example of an idle hook.  It is called on each
     cycle of the idle task.  It must *NOT* attempt to block.  In this case the
     idle task just queries the amount of FreeRTOS heap that remains.  See the
     memory management section on the http://www.FreeRTOS.org web site for memory
     management options.  If there is a lot of heap memory free then the
     configTOTAL_HEAP_SIZE value in FreeRTOSConfig.h can be reduced to free up
     RAM. */
    xFreeHeapSpace = xPortGetFreeHeapSize();

    /* Remove compiler warning about xFreeHeapSpace being set but never used. */
    (void) xFreeHeapSpace;
}
/******************************************************************************
 End of function  vApplicationIdleHook
 ******************************************************************************/

/******************************************************************************
 * Function Name: vAssertCalled
 * Description  : The FreeRTOS asser hook
 * Arguments    : IN  pcFile - Pointer to the file name
 *                IN  ulLine - The line number
 * Return Value : none
 ******************************************************************************/
void vAssertCalled (const char * pcFile, unsigned long ulLine)
{
    fprintf(stderr, "ASSERTION FAILED: FILE %s, LINE %d\r\n", pcFile, ulLine);
    abort();
}
/******************************************************************************
 End of function  vAssertCalled
 ******************************************************************************/

/******************************************************************************
 * Function Name: vApplicationTickHook
 * Description  : FreeRTOS application tick hook
 * Arguments    : none
 * Return Value : none
 ******************************************************************************/
void vApplicationTickHook (void)
{
    /* No function */
}
/******************************************************************************
 End of function  vApplicationTickHook
 ******************************************************************************/

//TODO: Stub out
void vParTestInitialise (void)
{
    /* Not Used */
}

/******************************************************************************
 End of function  vParTestInitialise
 ******************************************************************************/
//TODO: Stub out
void vParTestSetLED (unsigned long ulLED, signed long xValue)
{
    /* Not Used */
}

/******************************************************************************
 End of function  vParTestSetLED
 ******************************************************************************/
//TODO: Stub out
void vParTestToggleLED (unsigned long ulLED)
{
    /* Not Used */
}

/******************************************************************************
 End of function  vParTestToggleLED
 ******************************************************************************/

/******************************************************************************
 Private Function Prototypes
 ******************************************************************************/

/******************************************************************************
 Private global variables and functions
 ******************************************************************************/

/******************************************************************************
 End  Of File
 *******************************************************************************/
