/******************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only
 * intended for use with Renesas products. No other uses are authorized. This
 * software is owned by Renesas Electronics Corporation and is protected under
 * all applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
 * LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
 * ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
 * FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
 * ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software
 * and to discontinue the availability of this software. By using this software,
 * you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *******************************************************************************
 * Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
 *******************************************************************************
 * File Name    : cmdPlatform.c
 * Version      : 1.00
 * Device(s)    : Renesas
 * Tool-Chain   : N/A
 * OS           : N/A
 * H/W Platform : Renesas
 * Description  : The platform specific commands
 *******************************************************************************
 * History      : DD.MM.YYYY Ver. Description
 *              : 01.08.2009 1.00 First Release
 ******************************************************************************/

/******************************************************************************
 WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
 OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
 SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
 ******************************************************************************/

/******************************************************************************
 System Includes
 ******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>

/******************************************************************************
 User Includes
 ******************************************************************************/

#include "r_task.h"

/* Ensure local malloc is used */
#include "..\inc\malloc.h"
//#include "malloc.h"

#include "control.h"
#include "command.h"
#include "hwLED.h"
#include "sysUsage.h"
#include "iodefine.h"
#include "rzDmaTest.h"

/******************************************************************************
 External functions
 ******************************************************************************/

extern  void cmdShowDataRate(FILE    *pFile,
                             float   fTransferTime,
                             size_t  stLength);

/*****************************************************************************
 External Variables
 ******************************************************************************/

extern int_t           g_itimer;

/******************************************************************************
 Private Functions
 ******************************************************************************/

/******************************************************************************
Function Name: cmdMemoryPerformanceTest
Description:
Arguments:     IN  iArgCount The number of arguments in the argument list
               IN  ppszArgument - The argument list
               IN  pCom - Pointer to the command object
Return value:  CMD_OK for success
******************************************************************************/
static int cmdMemoryPerformanceTest(int     iArgCount,
                                    char  **ppszArgument,
                                    PCOMSET pCom)
{
    size_t stLength = (1024 * 1024);
    uint8_t * pbyTest;

    /* Get user specified length */
    if (iArgCount > 1)
    {
        float fTestSize = 1.0f;

        /* Get the desired file size */
        sscanf(ppszArgument[1], "%f", &fTestSize);

        /* Calculate the test file size - always a multiple of four bytes */
        stLength = (size_t) (fTestSize * 1024.0f * 1024.0f);

        /* Limit to a maximum size */
        if (stLength > (size_t) (1024.0f * 1024.0f * 10.0f))
        {
            stLength = (size_t) (1024.0f * 1024.0f * 10.0f);
        }

        /* Make a multiple of 4 bytes */
        stLength += (stLength % sizeof(uint32_t));
    }

    /* Allocate memory from SDRAM */
    pbyTest = (uint8_t *) exMalloc(HEAP_SDRAM, stLength);
    if (pbyTest)
    {
        float fTime;
        TMSTMP perfTimer;
        timerStartMeasurement(&perfTimer);
        memset(pbyTest, 0, stLength);
        fTime = timerStopMeasurement(&perfTimer);
        cmdShowDataRate(pCom->pOut, fTime, stLength);
        exFree(HEAP_SDRAM, pbyTest);
    }
    else
    {
        fprintf(pCom->pOut, "Failed to allocate memory for test\r\n");
        fflush(pCom->pOut);
    }

    return CMD_OK;
}
/******************************************************************************
 End of function  cmdMemoryPerformanceTest
 ******************************************************************************/

/******************************************************************************
 Function Name: cmdLed0
 Description:   Command to switch LED0 on / off
 Arguments:     IN  iArgCount The number of arguments in the argument list
 IN  ppszArgument - The argument list
 IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 ******************************************************************************/
static int cmdLed0 (int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    _Bool bfError = true;
    if (iArgCount > 1)
    {
        switch (*ppszArgument[1])
        {
            case '0' :
                led_off(LED0);
                bfError = false;
            break;

            case '1' :
                led_on(LED0);
                bfError = false;
            break;

            case '^' :
                led_toggle(LED0);
                bfError = false;
            break;
        }
    }
    if (bfError)
    {
        fprintf(pCom->pOut, "Expected 1, 0 or ^\r\n");
    }
    return CMD_OK;
}
/******************************************************************************
 End of function  cmdLed0
 ******************************************************************************/

/******************************************************************************
 Function Name: cmdLed1
 Description:   Command to switch LED1 on / off
 Arguments:     IN  iArgCount The number of arguments in the argument list
 IN  ppszArgument - The argument list
 IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 ******************************************************************************/
static int cmdLed1 (int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    _Bool bfError = true;
    if (iArgCount > 1)
    {
        switch (*ppszArgument[1])
        {
            case '0' :
                led_off(LED1);
                bfError = false;
            break;

            case '1' :
                led_on(LED1);
                bfError = false;
            break;

            case '^' :
                led_toggle(LED1);
                bfError = false;
            break;
        }
    }
    if (bfError)
    {
        fprintf(pCom->pOut, "Expected 1, 0 or ^\r\n");
    }
    return CMD_OK;
}
/******************************************************************************
 End of function  cmdLed1
 ******************************************************************************/

/******************************************************************************
 Function Name: cmdLed2
 Description:   Command to switch LED2 on / off
 Arguments:     IN  iArgCount The number of arguments in the argument list
 IN  ppszArgument - The argument list
 IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 ******************************************************************************/
static int cmdLed2 (int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    _Bool bfError = true;
    if (iArgCount > 1)
    {
        switch (*ppszArgument[1])
        {
            case '0' :
                led_off(LED2);
                bfError = false;
            break;

            case '1' :
                led_on(LED2);
                bfError = false;
            break;

            case '^' :
                led_toggle(LED2);
                bfError = false;
            break;
        }
    }
    if (bfError)
    {
        fprintf(pCom->pOut, "Expected 1, 0 or ^\r\n");
    }
    return CMD_OK;
}
/******************************************************************************
 End of function  cmdLed2
 ******************************************************************************/

/******************************************************************************
 Function Name: cmdLed3
 Description:   Command to switch LED3 on / off
 Arguments:     IN  iArgCount The number of arguments in the argument list
 IN  ppszArgument - The argument list
 IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 ******************************************************************************/
static int cmdLed3 (int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    _Bool bfError = true;
    if (iArgCount > 1)
    {
        switch (*ppszArgument[1])
        {
            case '0' :
                led_off(LED3);
                bfError = false;
            break;

            case '1' :
                led_on(LED3);
                bfError = false;
            break;

            case '^' :
                led_toggle(LED3);
                bfError = false;
            break;
        }
    }
    if (bfError)
    {
        fprintf(pCom->pOut, "Expected 1, 0 or ^\r\n");
    }
    return CMD_OK;
}
/******************************************************************************
 End of function  cmdLed3
 ******************************************************************************/

/******************************************************************************
 Function Name: cmdLed
 Description:   Command to switch LED 1..3 on / off
 Arguments:     IN  iArgCount The number of arguments in the argument list
 IN  ppszArgument - The argument list
 IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 ******************************************************************************/
static int cmdLed (int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    _Bool bfError = true;
    if (iArgCount > 1)
    {
        switch (*ppszArgument[1])
        {
            case '0' :
                cmdLed0(iArgCount - 1, &ppszArgument[1], pCom);
                bfError = false;
            break;

            case '1' :
                cmdLed1(iArgCount - 1, &ppszArgument[1], pCom);
                bfError = false;
            break;

            case '2' :
                cmdLed2(iArgCount - 1, &ppszArgument[1], pCom);
                bfError = false;
            break;

            case '3' :
                cmdLed3(iArgCount - 1, &ppszArgument[1], pCom);
                bfError = false;
            break;
        }
    }
    if (bfError)
    {
        fprintf(pCom->pOut, "Expected led number 1..3\r\n");
    }
    return CMD_OK;
}
/******************************************************************************
 End of function  cmdLed
 ******************************************************************************/

/******************************************************************************
 Function Name: cmdRestart
 Description:   Command to restart the system with a WDT reset
 Arguments:     IN  iArgCount The number of arguments in the argument list
 IN  ppszArgument - The argument list
 IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 ******************************************************************************/
static int cmdRestart (int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    AVOID_UNUSED_WARNING;

    /* Call the watch dog timer reset - All timer controls require a
     parameter - even if it is not used */
    control(g_itimer, CTL_WDT_RESET, ppszArgument);

    /* This code should never be executed */
    fprintf(pCom->pOut, "Restart failed!\r\n");
    return CMD_OK;
}
/******************************************************************************
 End of function  cmdRestart
 ******************************************************************************/

/******************************************************************************
 Function Name: dmaTest
 Description:   Function to perform the DMA test
 Arguments:     IN  srcMemory - The source memory type
 IN  destMemory - The destination memory type
 IN  stTransferSize - The length of memory to transfer
 IN  bfUseMemcpy - True to use memcpy in place of DMA transfer
 IN  iKeyboard - The file descriptor of the stream to read the
 terminating key press from
 IN  pFile - Pointer to a file stream to print status messages to
 Return value:  0 for success or -1 on error
 ******************************************************************************/
static int dmaTest(MEMTYPE  srcMemory,
                   MEMTYPE  destMemory,
                   size_t   stTransferSize,
                   bool_t   bfUseMemcpy,
                   int      iKeyboard,
                   FILE     *pFile)
{
    int_t iResult = 0;

    uint8_t ui_loop = (1);

    /* Allocate the source memory */
    void *pvSrc = exMalloc(srcMemory, stTransferSize + 1024UL);
    if (pvSrc)
    {
        /* Allocate a little extra on the end for thorough testing */
        void *pvDest = exMalloc(destMemory, stTransferSize + 1024UL);
        if (pvDest)
        {
            int iCount = 0;
            int ibond = 0;
            void *pvDest2, *pvDestToUse, *pvSrc2, *pvSrcToUse;
            pvDest2 = (void*) ((uint8_t*) pvDest + 1024UL);
            pvSrc2 = (void*) ((uint8_t*) pvSrc + 1024UL);
            while (ui_loop)
            {
                TMSTMP timeStamp;
                RZDMAE rzDmaError;
                float fTransferTime;
                const char * const pszMemoryTypes[] =
                {
                    "HEAP_WEB_SERVER",
                    "HEAP_LWIP_PRIMARY",
                    "HEAP_SDRAM",
                    "HEAP_SRAM",
                    "NUMBER_OF_HEAPS"
                };

                /* Alternate the destination by 1024 bytes during repeat testing
                 * to see if it exposes any problems
                 */
                iCount++;
                if (iCount & 1)
                {
                    pvDestToUse = pvDest;
                    pvSrcToUse = pvSrc;
                }
                else
                {
                    pvDestToUse = pvDest2;
                    pvSrcToUse = pvSrc2;
                }

                /* Check for using memcpy */
                if (bfUseMemcpy)
                {
                    fprintf(pFile,
                            "Memcpy %p (%s) -> %p (%s) length %lu\r\n",
                            pvSrcToUse, pszMemoryTypes[srcMemory],
                            pvDestToUse, pszMemoryTypes[destMemory],
                            (uint32_t)stTransferSize);
                }
                else
                {
                    fprintf(pFile,
                            "DMA %p (%s) -> %p (%s) length %lu\r\n",
                            pvSrcToUse, pszMemoryTypes[srcMemory],
                            pvDestToUse, pszMemoryTypes[destMemory],
                            (uint32_t)stTransferSize);
                }
                fflush(pFile);

                /* Start a measurement timer */
                timerStartMeasurement(&timeStamp);

                /* Perform the transfer */
                rzDmaError = rzDmaTest(pvDestToUse, pvSrcToUse, stTransferSize, bfUseMemcpy);

                /* Stop the measurement timer */
                fTransferTime = timerStopMeasurement(&timeStamp);

                /* Show the result of the transfer */
                fprintf(pFile, "Test %6d ",ibond++);

                /* Show the result of the transfer */
                switch (rzDmaError)
                {
                    case RZ_DMA_OK :
                    {
                        fprintf(pFile, "RZ_DMA_OK\r\n");
                        break;
                    }
                    case RZ_DMA_SRC_MEMORY_ADDRESS_ERROR :
                    {
                        fprintf(pFile, "RZ_DMA_SRC_MEMORY_ADDRESS_ERROR\r\n");
                        iKeyboard = -1;
                        iResult = -1;
                        break;
                    }
                    case RZ_DMA_DEST_MEMORY_ADDRESS_ERROR :
                    {
                        fprintf(pFile, "RZ_DMA_DEST_MEMORY_ADDRESS_ERROR\r\n");
                        iKeyboard = -1;
                        iResult = -1;
                        break;
                    }
                    case RZ_DMA_MEMORY_LENGTH_ERROR :
                    {
                        fprintf(pFile, "RZ_DMA_MEMORY_LENGTH_ERROR\r\n");
                        iKeyboard = -1;
                        iResult = -1;
                        break;
                    }
                    case RZ_DMA_MEMORY_BOUNDARY_ERROR :
                    {
                        fprintf(pFile, "RZ_DMA_MEMORY_BOUNDARY_ERROR\r\n");
                        iKeyboard = -1;
                        iResult = -1;
                        break;
                    }
                    case RZ_DMA_MEMORY_COMPARE_ERROR :
                    {
                        fprintf(pFile, "RZ_DMA_MEMORY_COMPARE_ERROR\r\n");
                        iKeyboard = -1;
                        break;
                    }
                    default :
                    {
                        fprintf(pFile, "Unknown return code\r\n");
                        iKeyboard = -1;
                        break;
                    }
                }

                /* Show the transfer rate */
                cmdShowDataRate(pFile, fTransferTime, stTransferSize);

                /* Check to see if the test is in repeat */
                if ((-1) != iKeyboard)
                {
                    /* Stop when a key is pressed */
                    if (control(iKeyboard, CTL_GET_RX_BUFFER_COUNT, NULL) > 0)
                    {
                        /* End while loop */
                        ui_loop = 0;
                    }
                }
                else
                {
                    /* End while loop */
                    ui_loop = 0;
                }
            } /* End of While Loop */

            exFree(destMemory, pvDest);
        }
        else
        {
            fprintf(pFile, "Failed to allocate destination memory\r\n");
            iResult = -1;
        }
        exFree(srcMemory, pvSrc);
    }
    else
    {
        fprintf(pFile, "Failed to allocate source memory\r\n");
        iResult = -1;
    }

    return iResult;
}
/******************************************************************************
 End of function  dmaTest
 ******************************************************************************/

/******************************************************************************
 Function Name: cmdDmaTest
 Description:   Command to test a DMA transfer
 Arguments:     IN  iArgCount The number of arguments in the argument list
 IN  ppszArgument - The argument list
 IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 ******************************************************************************/
static int cmdDmaTest (int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    if (iArgCount > 1)
    {
        /* Set the defaults for the test */
        MEMTYPE srcMemory = HEAP_SRAM;
        MEMTYPE destMemory = HEAP_SRAM;
        size_t stTransferSize = 1024UL * 256UL * 1UL;
        bool_t bfUseMemcpy = false;
        bool_t b_error = false;
        int_t  iKeyboard = -1;
        int_t  iArgument = 1;
        while ((iArgument < iArgCount) && (false == b_error))
        {
            /* Format of arguments -a /c \s1024 ... */
            if (('-'  == ppszArgument[iArgument][0]) ||
                ('\\' == ppszArgument[iArgument][0]) ||
                ('/'  == ppszArgument[iArgument][0]) )
            {
                switch (ppszArgument[iArgument][1])
                {
                    case 'a' :
                    {
                        srcMemory = HEAP_SDRAM;
                        break;
                    }
                    case 'b' :
                    {
                        destMemory = HEAP_SDRAM;
                        break;
                    }
                    case 'r' :
                    {
                        iKeyboard = filePointerToDescriptor(pCom->pOut);
                        break;
                    }
                    case 's' :
                    {
                        sscanf(&ppszArgument[iArgument][2], "%lu",
                              (uint32_t *)&stTransferSize);
                        break;
                    }
                    case 'm' :
                    {
                        bfUseMemcpy = true;
                        break;
                    }
                    case 'x' :
                    {
                        break;
                    }
                    default :
                    {
                        fprintf(pCom->pOut, "Unknown option %s\r\n", ppszArgument[iArgument]);
                        break;
                    }
                }
            }
            else
            {
                fprintf(pCom->pOut, "Invalid option %s\r\n", ppszArgument[iArgument]);

                /* Force while loop to end */
                b_error = true;
            }
            iArgument++;
        } /* end of while loop */

        /* Call the DAM test function */
        if ((dmaTest(srcMemory,
                     destMemory,
                     stTransferSize,
                     bfUseMemcpy,
                     iKeyboard,
                     pCom->pOut) == 0)

            /* If a key was pressed to stop the test */
            && ((-1) != iKeyboard))
        {
            /* Read the key to prevent it appearing on the command line */
            fgetc(pCom->pIn);
        }

    }
    else
    {
        int iTable = 0;

        /* Find the help string and print it */
        while (iTable < pCom->iNumTables)
        {
            int iCommand = 0;
            PCMDFNASS pCommand = pCom->ppFn[iTable]->pCommandList;
            while (iCommand < pCom->ppFn[iTable]->dwNumberOfCommands)
            {
                if ((pCommand->pFunction == cmdDmaTest)
                &&  (pCommand->pszCommandDescription))
                {
                    fprintf(pCom->pOut, "%s %s\r\n",
                           pCommand->pszCommand,
                           pCommand->pszCommandDescription);
                }
                pCommand++;
                iCommand++;
            }
            iTable++;
        }
    }
    fflush(pCom->pOut);
    return CMD_OK;
}
/******************************************************************************
 End of function  cmdDmaTest
 ******************************************************************************/

/* Table that associates command letters, function pointer and a little
 description of what the command does */
const CMDFNASS gpcmdPlatform[] =
{
    {
        "mperf",
        cmdMemoryPerformanceTest,
        "<CR> - Check the speed of writing to SDRAM"
    },

    {
        "led",
        cmdLed,
        "n s <CR> - Sets LED n state to On s = 1,\r\n"
        "             Off s = 0, Toggle s = ^"
    },

    {
        "led0",
        cmdLed0,
        "s <CR> - Sets LED0 state to On s = 1,\r\n"
        "              Off s = 0, Toggle s = ^"
    },

    {
        "led1",
        cmdLed1,
        "s <CR> - Sets LED1 state to On s = 1,\r\n"
        "              Off s = 0, Toggle s = ^"
    },

    {
        "led2",
        cmdLed2,
        "s <CR> - Sets LED2 state to On s = 1,\r\n"
        "              Off s = 0, Toggle s = ^"
    },

    {
        "led3",
        cmdLed3,
        "s <CR> - Sets LED3 state to On s = 1,\r\n"
        "              Off s = 0, Toggle s = ^"
    },

    {
        "restart",
        cmdRestart,
        "<CR> - Restart the system with a WDT reset"
    },

    {
        "dma",
        cmdDmaTest,
        "-o<CR> where o is one of the or more following options\r\n"
        "    -r = repeat test until key press\r\n"
        "    -s = transfer size (default 256KByte)\r\n"
        "    -a = source in SDRAM\r\n"
        "    -b = destination in SDRAM\r\n"
        "    -m = use C memcpy function in place of the DMA\r\n"
        "    -x = execute\r\n"
        "    (no option) = Command list display\r\n"
    }
};

/* Table that points to the above table and contains the number of entries */
CMDTAB gcmdPlatform =
{
    gpcmdPlatform,
    (sizeof(gpcmdPlatform)/sizeof(CMDFNASS))
};

/******************************************************************************
End  Of File
******************************************************************************/
