/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : command.c
* Version      : 1.01
* Device(s)    : Renesas
* Tool-Chain   : N/A
* OS           : N/A
* H/W Platform : Renesas
* Description  : The entry point of the main command handler and associated
*                commands
*******************************************************************************
* History      : DD.MM.YYYY Version Description
*              : 04.02.2010 1.00    First Release
*              : 10.06.2010 1.01    Updated type definitions
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "command.h"
#include "version.h"
#include "devLink.h"
#include "r_task.h"
#include "nonVolatileData.h"
#include "r_typedefs.h"

/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/* Define the number of characters printed on a line for the "mem" command */
#define MEM_COMMAND_CHAR_COUNT      16
#define LOGIN_AND_PASSWORD_STRING   32

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/* The Mass storage commands */
extern const CMDTAB gcmdPlatform;
extern int_t g_irtc;

/******************************************************************************
 Exported global variables and functions (to be accessed by other files)
 ******************************************************************************/

/******************************************************************************
 Private global variables and functions
 ******************************************************************************/
/* Terminal window escape equences */
static const char * const gpszClearScreen = "\x1b[2J";
static const char * const gpszCursorHome = "\x1b[H";

uint8_t * cmdViewMemory (uint8_t *pbyView, uint8_t bLength, FILE *pOut);
static _Bool cmdAuthenticateUser (PCOMSET pCom, PCMDERR pCmdErr);

/* Define the engineering exp symbols for Kilo, Mega, Giga, Tera, Peta,
 Exa, Zetta and Yotta */
static char * const gpszEngMultChars = " kMGTPEZY";
const CMDTAB gcmdCommand;

/* Define the command tables required */
PCMDTAB   gppCommands[] =
{
    (PCMDTAB)&gcmdCommand,
    (PCMDTAB)&gcmdPlatform,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
    (PCMDTAB)NULL,
};
int32_t   giNumCommands = (sizeof(gppCommands)) / sizeof(PCMDTAB);

/* The default user name is admin and the password is password */
static const char gpszDefaultUser[] = "admin:��������";

/*****************************************************************************
Global Variables
******************************************************************************/

char gpszLoginsAndPasswords[LOGIN_AND_PASSWORD_STRING];
_Bool  gbfLoginsLoaded = false;

/******************************************************************************
Public Functions
******************************************************************************/

/******************************************************************************
Function Name: CommandAddCmdTab
Description:   Function to insert xupplimental commandlines to the main console
Arguments:     IN  pIn - Pointer to the input file stream
Return value:  0 = ERROR
               1 = handle to command line (use when removal is required)
******************************************************************************/
int32_t CommandAddCmdTab(CMDTAB *cmd)
{
    int32_t pos  = 0;
    int32_t loop = 0;

    for(loop = 0; loop < giNumCommands; loop++)
    {
        if(NULL == gppCommands[loop])
        {
            gppCommands[loop] = cmd;
            pos  = loop;
            loop = giNumCommands;
        }
    }
    return pos;
}
/******************************************************************************
End of function  CommandAddCmdTab
******************************************************************************/

/******************************************************************************
Function Name: cmdConsole
Description:   Function to insert supplemental commands to the main console
Arguments:     IN  pIn - Pointer to the input file stream
               IN  pOut - Pointer to the file stream for echo of command input
               IN  pszPrompt - Pointer to a command prompt
Return value:  none
******************************************************************************/
void cmdConsole(FILE* pIn, FILE *pOut, char *pszPrompt)
{
    PCOMSET pCom = malloc(sizeof(COMSET));
    int32_t pos;

    giNumCommands = (sizeof(gppCommands)) / sizeof(PCMDTAB);

    /* remove unused console command lists */
    for(pos = (giNumCommands-1); pos > 0; pos--)
    {
        if(NULL == gppCommands[pos])
        {
            giNumCommands--;
        }
    }

    if (pCom)
    {
        while (1)
        {
            CMDERR cmdErr = CMD_OK;

            /* Print a welcome message */
            fprintf(pOut,
                    WELCOME_MSG,
                    gpszClearScreen,
                    gpszCursorHome,
                    WEB_ENGINE_VERSION,
                    WEB_ENGINE_RELEASE,
                    WEB_ENGINE_BUILD);
            fflush(pOut);

            /* Initialise the console for login */
            memset(pCom, 0, sizeof(COMSET));
            pCom->pIn = pIn;
            pCom->pOut = pOut;

            /* Authenticate the user */
            if (cmdAuthenticateUser(pCom, &cmdErr))
            {
                /* Process commands until the link is lost or the command
                   returns a code greater than CMD_ERROR_IN_IO */
                while (cmdErr < CMD_USER_EXIT)
                {
                    cmdErr = Console(pCom,
                                     (void*)gppCommands,
                                     giNumCommands,
                                     pIn,
                                     pOut,
                                     pszPrompt);
                }
            }

            /* CMD_ERROR_IN_IO is returned when the client disconnects
               check for an IP stream */
            if ((CMD_ERROR_IN_IO == cmdErr)
            &&  (control(filePointerToDescriptor(pCom->pIn),
                         CTL_STREAM_TCP,
                         NULL) == 0))
            {
                /* Log out */
                cmdErr = CMD_LOG_OUT;
            }

            /* Check for the "exit<CR>" command return code */
            if (CMD_LOG_OUT == cmdErr)
            {
                fprintf(pCom->pOut, "\r\nBye!\r\n");

                /* Do "exit" code */
                free(pCom);
                return;
            }
        }
    }
}
/******************************************************************************
End of function  cmdConsole
******************************************************************************/

/******************************************************************************
Function Name: cmdViewMemory
Description:   Function to show memory in format
               XX XX XX XX XX XX XX XX : AAAAAAAAAAAAAAAAAA
               where XX is a H rep & AA is ascii rep of byte
Arguments:     IN  pbyView - pointer to the memory to diaplay
               IN  bLength - The number of bytes to display
               OUT pOut = Pointer to the file stream to print to
Return value:  address of last printed byte
******************************************************************************/
uint8_t * cmdViewMemory(uint8_t *pbyView, uint8_t bLength, FILE *pOut)
{
    uint8_t byCount, byData;
    fprintf(pOut, "%.8lX  ", (uint32_t)pbyView);

    /* Do hex segment */
    for (byCount = (uint8_t) 0; byCount < bLength; byCount++)
    {
        fprintf(pOut, "%.2X ", *(pbyView + byCount));
    }

    /* Print separator */
    fprintf(pOut, " : ");

    /* Do ASCII segment */
    for (byCount = (uint8_t) 0; byCount < bLength; byCount++)
    {
        byData = *(pbyView + byCount);

        /* Print . for non-ascii data */
        if ((byData < 0x7F) && (byData >= ' '))
        {
            fprintf(pOut, "%c", byData);
        }
        else
        {
            fprintf(pOut, ".");
        }
    }
    fprintf(pOut, "\r\n");

    return (pbyView + byCount);
}
/******************************************************************************
 End of function cmdViewMemory
 ******************************************************************************/

/******************************************************************************
 Function Name: cmdShowDataRate
 Description:   Function to print the data transfer rate (in engineering units)
 Arguments:     IN  pFile - Pointer to the file to print to
 IN  fTransferTime - The transfer time in seconds
 IN  stLength - The length of data transferred
 Return value:  none
 ******************************************************************************/
void cmdShowDataRate (FILE *pFile, float fTransferTime, size_t stLength)
{
    if (fTransferTime > 0.0f)
    {
        float fTransferRate;
        char * pszEngMult = gpszEngMultChars;

        /* Calculate the transfer rate */
        fTransferRate = (float) stLength / fTransferTime;

        /* Scale for engineering multiplier */
        while (fTransferRate > 1000.0f)
        {
            fTransferRate /= 1000.0f;
            pszEngMult++;
        }

        /* Print the detailes */
        fprintf(pFile,
                "Transferred %lu bytes at %.2f %c Bytes per Second\r\n",
                (uint32_t)stLength,
                fTransferRate,
                *pszEngMult);
    }
    else
    {
        fprintf(pFile, "Invalid transfer time\r\n");
    }
    fflush(pFile);
}
/******************************************************************************
 End of function  cmdShowDataRate
 ******************************************************************************/

/*****************************************************************************
 Function Name: cmdShowMemorySize
 Description:   Function to print a memory size in engineering units
 Arguments:     IN  pFile - Pointer to the file to print to
 IN  stLength - The length of the memoy
 Return value:  none
 *****************************************************************************/
void cmdShowMemorySize (FILE *pFile, size_t stLength)
{
    float fSize = (float) stLength;
    char * pszEngMult = gpszEngMultChars;

    /* Scale for engineering multiplier */
    while (fSize > 1024.0f)
    {
        fSize /= 1024.0f;
        pszEngMult++;
    }

    /* Print the details */
    fprintf(pFile, "%.3f%cB", fSize, *pszEngMult);
    fflush(pFile);
}
/*****************************************************************************
End of function  cmdShowMemorySize
******************************************************************************/

/******************************************************************************
Function Name: cmdShowBin
Description:   Command to show memory in the format
               XX XX XX XX XX XX XX XX : AAAAAAAAAAAAAAAAAA
               where XX is a H rep & AA is ascii rep of byte
Arguments:     IN  pbyView - pointer to the memory to diaplay
               IN  iLength - The number of bytes to display
               IN  stOffset - The file offset
               OUT pOut - Pointer to the file stream to print to
Return value:  address of last printed byte
******************************************************************************/
uint8_t * cmdShowBin(uint8_t *pbyView, int iLength, size_t stOffset, FILE *pOut)
{
    int iCount;
    fprintf(pOut, "%.8lX  ", (uint32_t)stOffset);

    /* Do hex segment */
    for (iCount = 0; iCount < iLength; iCount++)
    {
        fprintf(pOut, "%.2X ", *(pbyView + iCount));
    }

    /* Print separator */
    fprintf(pOut, " : ");

    /* Do ASCII segment */
    for (iCount = 0; iCount < iLength; iCount++)
    {
        uint8_t byData = *(pbyView + iCount);

        /* Print . for non-ascii data */
        if ((byData < 0x7F) && (byData >= ' '))
        {
            fprintf(pOut, "%c", byData);
        }
        else
        {
            fprintf(pOut, ".");
        }
    }
    fprintf(pOut, "\r\n");
    fflush(pOut);
    return (pbyView + iCount);
}
/******************************************************************************
End of function cmdShowBin
******************************************************************************/

/*****************************************************************************
Function Name: cmdLoadLogin
Description:   Function to load the login information from EEROM
Arguments:     none
Return value:  none
*****************************************************************************/
static void cmdLoadLogin(void)
{
    if (!gbfLoginsLoaded)
    {
        /* Load the logins and passwords */
        if (nvLoad(NVDT_LOGINS_AND_PASSWORDS_V1,
                   gpszLoginsAndPasswords,
                   sizeof(gpszLoginsAndPasswords)))
        {
            /* When there is no user information in the non volatile memory then
               admin and password will be default */
            strcpy(gpszLoginsAndPasswords, gpszDefaultUser);
        }
        gbfLoginsLoaded = true;
    }
}
/*****************************************************************************
End of function  cmdLoadLogin
******************************************************************************/

/*****************************************************************************
Function Name: cmdCheckUserNameAndPassword
Description:   Function to check the user name and password
Arguments:     IN  pszUserName - Pointer to the user name string
               IN  pszPassword - Pointer to the password
Return value:  true if the user name and password are correct
*****************************************************************************/
_Bool cmdCheckUserNameAndPassword(char *pszUserName,
                                  char *pszPassword)
{
    char *pszLogin;
    cmdLoadLogin();

    /* Put the delimiter on the end of the user name to make the login */
    strcat(pszUserName, ":");

    /* Look for the user name */
    pszLogin = strstr(gpszLoginsAndPasswords, pszUserName);
    if (pszLogin)
    {
        bool_t b_stop = false;
        char pszBuffer[LOGIN_AND_PASSWORD_STRING];
        char *pszPass = pszLogin + strlen(pszUserName);
        char *pszTemp = pszPass;
        char *pszDest = pszBuffer;
        memset(pszBuffer, 0, sizeof(pszBuffer));

        /* Replace the delimiter with a terminator & decypher the password */
        while ((*pszTemp) && (false == b_stop))
        {
            if ((!*pszTemp) || ('|' == *pszTemp))
            {
                *pszDest = '\0';
                b_stop = true;
            }
            else
            {
                *pszDest = (char) (*pszTemp ^ 129);
                pszTemp++;
                pszDest++;
            }
        } /* end of while loop */
        if (strcmp(pszPassword, pszBuffer) == 0)
        {
            return true;
        }
    }
    return false;
}
/*****************************************************************************
 End of function  cmdCheckUserNameAndPassword
 ******************************************************************************/

/******************************************************************************
 Private Functions
 ******************************************************************************/

/*****************************************************************************
 Function Name: cmdGetInput
 Description:   Function to get input from the user up to the new-line character
 Arguments:     IN  pCom - Pointer to the command object
 Return value:  0 for success otherwise error code
 *****************************************************************************/
static CMDERR cmdGetInput (PCOMSET pCom)
{
    FILE *pIn = pCom->pIn;
    pCom->va.uiBufIdx = 0;
    while (true)
    {
        /* Read a character from the input stream */
        pCom->va.iData = fgetc(pIn);

        /* Check for error */
        if (pCom->va.iData < 0)
        {
            clearerr(pIn);
            return CMD_ERROR_IN_IO;
        }
        else
        {
            switch (pCom->va.iData)
            {
                case '\t' :
                case '\r' :
                {
                    /* Terminate the string */
                    pCom->va.pszBuffer[pCom->va.uiBufIdx] = (uint8_t) '\0';
                    return CMD_OK;
                }

                /* Prevent escape sequences starting */
                case CMD_ESCAPE_CHARACTER :
                {
                    break;
                }
                default :
                {
                    _Bool bfCommand = false;
                    conProcessChar(pCom, (char) pCom->va.iData, &bfCommand);
                    fflush(pCom->pOut);
                    break;
                }
            }
        }
    }
    return CMD_ESCAPE_SEQUENCE;
}
/*****************************************************************************
End of function  cmdGetInput
******************************************************************************/

/*****************************************************************************
Function Name: cmdAuthenticateUser
Description:   Function to authenticate a user
Arguments:     IN  pCom - Pointer to the command object
               OUT pCmdErr - Pointer to the error status
Return value:  True if the user has been authenticated
*****************************************************************************/
static _Bool cmdAuthenticateUser(PCOMSET pCom, PCMDERR pCmdErr)
{
    *pCmdErr = CMD_OK;
    if (control(filePointerToDescriptor(pCom->pIn),
                CTL_STREAM_REQUIRES_AUTHENTICATION,
                NULL) == 0)
    {
        char  pszUserName[CMD_READER_LINE_SIZE];
        FILE    *pOut = pCom->pOut;
        CMDERR  cmdErr;
        /* The CTL_STREAM_REQUIRES_AUTHENTICATION control succeeded which
           means we should ask for Login and Password */
        do
        {
            fprintf(pOut, "\r\nLogin: ");
            fflush(pOut);

            /* Get the user name */
            cmdErr = cmdGetInput(pCom);
            if (cmdErr)
            {
                *pCmdErr = cmdErr;
                return false;
            }

            /* The login name must be at least 1 character long */
        } while (pCom->va.uiBufIdx < 1);

        /* Copy to the destination */
        strcpy(pszUserName, pCom->va.pszBuffer);
        fprintf(pOut, "\r\nPassword: ");
        fflush(pOut);

        /* Don't echo the password characters */
        pCom->bfSecrecy = true;

        /* Get the user name */
        cmdErr = cmdGetInput(pCom);
        pCom->bfSecrecy = false;
        if (cmdErr)
        {
            *pCmdErr = cmdErr;
            return false;
        }
        fprintf(pOut, "\r\n");
        fflush(pOut);

        /* Check the user's details */
        return cmdCheckUserNameAndPassword(pszUserName, pCom->va.pszBuffer);
    }
    else
    {
        /* The stream does not require authentication so skip this */
        return true;
    }
}
/*****************************************************************************
End of function  cmdAuthenticateUser
******************************************************************************/

/******************************************************************************
Function Name: cmdSetDate
Description:   Command to show and set the date
Arguments:     IN  iArgCount The number of arguments in the argument list
               IN  ppszArgument - The argument list
               IN  pCom - Pointer to the command object
Return value:  CMD_OK for success
******************************************************************************/
static int cmdSetDate(int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    DATE    Date;
    if (control(g_irtc, CTL_GET_DATE, &Date) == 0)
    {
        static char * const ppszWeekDay[] =
        {
            "Monday",
            "Tuesday",
            "Wednesday",
            "Thursday",
            "Friday",
            "Saturday",
            "Sunday"
        };
        static char * const ppszMonth[] =
        {
            "None",
            "January",
            "February",
            "March",
            "April",
            "May",
            "June",
            "July",
            "August",
            "September",
            "October",
            "November",
            "December"
        };
        if (iArgCount >= 2)
        {
            int     chDelim;
            int     iDay = (int)Date.Field.Day;
            int     iMonth = (int)Date.Field.Month;
            int     iYear = (int)Date.Field.Year;

            if (sscanf(ppszArgument[1], "%d%c%d%c%d",
                       &iDay, (char *)&chDelim, &iMonth, (char *)&chDelim, &iYear) != -1)
            {
                Date.Field.Day = iDay;
                Date.Field.Month = iMonth;
                Date.Field.Year = iYear;
                if (control(g_irtc, CTL_SET_DATE, &Date))
                {
                    fprintf(pCom->pOut, "Failed to set date\r\n");
                }
            }
        }

        /* Get the date again */
        control(g_irtc, CTL_GET_DATE, &Date);

        /* Print the date and time */
        fprintf(pCom->pOut, "%s %.2d %s %.4d\r\n",
               ppszWeekDay[(int)Date.Field.WeekDay],
               (int)Date.Field.Day,
               ppszMonth[(int)Date.Field.Month],
               (int)Date.Field.Year);
    }
    else
    {
        fprintf(pCom->pOut, "Failed to get date\r\n");
    }

    return CMD_OK;
}
/******************************************************************************
 End of function  cmdSetDate
 ******************************************************************************/

/******************************************************************************
 Function Name: cmdSetTime
 Description:   Command to show and set the time
 Arguments:     IN  iArgCount The number of arguments in the argument list
 IN  ppszArgument - The argument list
 IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 ******************************************************************************/
static int cmdSetTime (int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    DATE    Date;
    if (control(g_irtc, CTL_GET_DATE, &Date) == 0)
    {
        if (iArgCount >= 2)
        {
            char    chDelim;
            int     iHour = (int)Date.Field.Hour;
            int     iMinute = (int)Date.Field.Minute;
            int     iSecond = (int)Date.Field.Second;

            if (sscanf(ppszArgument[1], "%d%c%d%c%d",
                       &iHour, &chDelim, &iMinute, &chDelim, &iSecond) != -1)
            {
                Date.Field.Hour = iHour;
                Date.Field.Minute = iMinute;
                Date.Field.Second = iSecond;
                if (control(g_irtc, CTL_SET_DATE, &Date))
                {
                    fprintf(pCom->pOut, "Failed to set time\r\n");
                }
            }
        }

        /* Get the time again */
        control(g_irtc, CTL_GET_DATE, &Date);

        /* Print the date and time */
        fprintf(pCom->pOut, "%.2d:%.2d:%.2d\r\n",
               (int)Date.Field.Hour,
               (int)Date.Field.Minute,
               (int)Date.Field.Second);
    }
    else
    {
        fprintf(pCom->pOut, "Failed to get time\r\n");
    }

    return CMD_OK;
}
/******************************************************************************
End of function  cmdSetTime
******************************************************************************/

/******************************************************************************
 Function Name: cmdReadMemory
 Description:   Command to view memory
 Arguments:     IN  iArgCount The number of arguments in the argument list
 IN  ppszArgument - The argument list
 IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 ******************************************************************************/
static int cmdReadMemory (int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    uint32_t    dwTotal = 21;
    uint8_t     *pbyMemory = 0, *pbyEnd;
    AVOID_UNUSED_WARNING;

    /* Get address from command line */
    sscanf(ppszArgument[1], "%lX", (uint32_t *)&pbyMemory);

    /* Get length from the command line */
    sscanf(ppszArgument[2], "%lX", &dwTotal);

    /* Show what we are reading */
    fprintf(pCom->pOut, "View memory Address = H'%lX, Lines = H'%lX\r\n",
            (uint32_t)pbyMemory, (uint32_t)dwTotal);
    /* calculate the last memory address */
    pbyEnd = pbyMemory + (dwTotal * MEM_COMMAND_CHAR_COUNT);
    while (pbyMemory < pbyEnd)
    {
        /* Show the memory */
        pbyMemory = cmdViewMemory(pbyMemory, MEM_COMMAND_CHAR_COUNT, pCom->pOut);

    }
    return CMD_OK;
}
/******************************************************************************
 End of function cmdReadMemory
 ******************************************************************************/

/*****************************************************************************
 Function Name: cmdUserName
 Description:   Command to change the user name
 Arguments:     IN  iArgCount The number of arguments in the argument list
 IN  ppszArgument - The argument list
 IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 *****************************************************************************/
int cmdUserName (int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    if (3 == iArgCount)
    {
        char *pszLogin;
        size_t stLoginLen;

        /* Load the logins and passwords */
        cmdLoadLogin();
        stLoginLen = strlen(ppszArgument[1]);
        strcat(ppszArgument[1], ":");

        /* Look for the user name */
        pszLogin = strstr(gpszLoginsAndPasswords, ppszArgument[1]);
        if (pszLogin)
        {
            char pszNewLoginsAndPasswords[LOGIN_AND_PASSWORD_STRING];
            char *pszSrc = gpszLoginsAndPasswords;
            char *pszDest = pszNewLoginsAndPasswords;
            char *pszNewLogin = ppszArgument[2];
            memset(pszNewLoginsAndPasswords, 0,
                   sizeof(pszNewLoginsAndPasswords));
            /* Copy any data in the string before the login */
            while (pszSrc < pszLogin)
            {
                *pszDest++ = *pszSrc++;
            }

            /* Copy in the new user name */
            while (*pszNewLogin)
            {
                *pszDest++ = *pszNewLogin++;
            }

            /* Copy in the remainder of the string */
            pszSrc += stLoginLen;
            while (*pszSrc)
            {
                *pszDest++ = *pszSrc++;
            }
            if (!nvStore(NVDT_LOGINS_AND_PASSWORDS_V1,
                         pszNewLoginsAndPasswords,
                         sizeof(gpszLoginsAndPasswords)))
            {
                memcpy(gpszLoginsAndPasswords,
                       pszNewLoginsAndPasswords,
                       sizeof(gpszLoginsAndPasswords));
                if (pCom)
                {
                    fprintf(pCom->pOut, "OK\r\n");
                }
            }
            else if (pCom)
            {
                fprintf(pCom->pOut, "**Error: Failed to store logins and passwords\r\n");
            }
        }
        else if (pCom)
        {
            fprintf(pCom->pOut, "Invalid user name\r\n");
        }
    }
    else if (pCom)
    {
        fprintf(pCom->pOut, "Expecting <old_user_name> <new_user_name>\r\n");
    }
    return CMD_OK;
}
/*****************************************************************************
 End of function  cmdUserName
 ******************************************************************************/

/*****************************************************************************
 Function Name: cmdChangePassword
 Description:   Command to change the user name
 Arguments:     IN  iArgCount The number of arguments in the argument list
 IN  ppszArgument - The argument list
 IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 *****************************************************************************/
int cmdChangePassword (int iArgCount, char **ppszArgument, PCOMSET pCom)
{
//TODO: List arguments and order somewhere
    /* Only accept a call with 4 arguments */
    if ((4 == iArgCount) && (cmdCheckUserNameAndPassword(ppszArgument[1],
                                                         ppszArgument[2])))
    {
        char *pszLogin;
        size_t stLoginLen;

        /* Load the logins and passwords */
        stLoginLen = strlen(ppszArgument[1]);

        /* Look for the user name */
        pszLogin = strstr(gpszLoginsAndPasswords, ppszArgument[1]);
        if (pszLogin)
        {
            bool_t b_stop = false;
            char pszNewLoginsAndPasswords[LOGIN_AND_PASSWORD_STRING];
            char *pszSrc = gpszLoginsAndPasswords;
            char *pszDest = pszNewLoginsAndPasswords;
            char *pszNewPassword = ppszArgument[3];
            memset(pszNewLoginsAndPasswords, 0,
                   sizeof(pszNewLoginsAndPasswords));
            /* Copy any data until the end of the login */
            pszLogin += stLoginLen;
            while (pszSrc < pszLogin)
            {
                *pszDest++ = *pszSrc++;
            }

            /* Copy in the new password */
            while (*pszNewPassword)
            {
                *pszDest++ = (uint8_t) (*pszNewPassword++ ^ 129);
            }

            /* Point to the end of the old password */
            pszSrc += stLoginLen;
            while ((*pszSrc) && (false == b_stop))
            {
                if ('|' == *pszSrc)
                {
                    b_stop = true;
                }
                else
                {
                    pszSrc++;
                }
            }

            /* Copy in the remainder of the string */
            while (*pszSrc)
            {
                *pszDest++ = *pszSrc++;
            }
            if (!nvStore(NVDT_LOGINS_AND_PASSWORDS_V1,
                         pszNewLoginsAndPasswords,
                         sizeof(gpszLoginsAndPasswords)))
            {
                memcpy(gpszLoginsAndPasswords,
                       pszNewLoginsAndPasswords,
                       sizeof(gpszLoginsAndPasswords));
                if (pCom)
                {
                    fprintf(pCom->pOut, "OK\r\n");
                }
            }
            else if (pCom)
            {
                fprintf(pCom->pOut,
                        "**Error: Failed to store logins and passwords\r\n");
            }
        }
        else if (pCom)
        {
            fprintf(pCom->pOut, "Invalid user name\r\n");
        }
    }
    else if (pCom)
    {
        fprintf(pCom->pOut,
                "Expecting <user_name> <password> <new_password>\r\n");
    }
    return CMD_OK;
}
/*****************************************************************************
End of function  cmdChangePassword
******************************************************************************/

/******************************************************************************
Function Name: cmdSystemUsage
Description:   Function to show the heap and event usage information
Arguments:     IN  iArgCount The number of arguments in the argument list
               IN  ppszArgument - The argument list
               IN  pCom - Pointer to the command object
Return value:  0 for success otherwise error code
******************************************************************************/
static int cmdSystemUsage(int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    SYSUSAGE    sysUsage;
    HEAPUSAGE   memUsage;
    MEMTYPE     memType;
    AVOID_UNUSED_WARNING;

    /* For each heap */
    for (memType = 0; memType < NUMBER_OF_HEAPS; memType++)
    {
        /* Get the memory usage information */
        heapUsage(&memUsage, memType);

        /* Print the heap number */
        fprintf(pCom->pOut, "Heap %d:\r\n", memType);

        /* Print the heap usage */
        fprintf(pCom->pOut, "    Free ");
        cmdShowMemorySize(pCom->pOut, memUsage.stTotalFree);
        fprintf(pCom->pOut, ", Used ");
        cmdShowMemorySize(pCom->pOut, memUsage.stTotalAllocated);
        fprintf(pCom->pOut, ", Total ");
        cmdShowMemorySize(pCom->pOut, memUsage.stTotalHeap);
        fprintf(pCom->pOut, "\r\n    Largest free segment ");
        cmdShowMemorySize(pCom->pOut, memUsage.stLargestFreeSegment);
        fprintf(pCom->pOut, ", Overhead ");
        cmdShowMemorySize(pCom->pOut, memUsage.stOverhead);
        fprintf(pCom->pOut, "\r\n");

        /* Print the fragmentation */
        fprintf(pCom->pOut, "    Fragmentation %.3f%%\r\n",
                (float)((1.0f - ((float)memUsage.stLargestFreeSegment /
                                 (float)memUsage.stTotalFree)) * 100));
    }

    /* Get the system usage information */
    systemUsage(&sysUsage);

    /* Print the results */
    fprintf(pCom->pOut, "System:\r\n    Files %lu/%lu\r\n"
            "    Events %lu/%lu\r\n    Timers %lu/%lu\r\n",
            (uint32_t)sysUsage.stFilesInUse,
            (uint32_t)sysUsage.stFileTotal,
            (uint32_t)sysUsage.stEventsInUse,
            (uint32_t)sysUsage.stEventTotal,
            (uint32_t)sysUsage.stTimersInUse,
            (uint32_t)sysUsage.stTimerTotal);
    return CMD_OK;
}
/******************************************************************************
End of function  cmdSystemUsage
******************************************************************************/

/*****************************************************************************
Function Name: cmdListTasks
Description:   Command to list all the running tasks
Arguments:     IN  iArgCount The number of arguments in the argument list
               IN  ppszArgument - The argument list
               IN  pCom - Pointer to the command object
Return value:  0 for success otherwise error code
*****************************************************************************/
static int cmdListTasks(int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    /* Get the task information string */
    char *pszTaskInformation = taskInformation();
    AVOID_UNUSED_WARNING;
    if (pszTaskInformation)
    {
        /* Write it to the output file */
        size_t stLength = strlen(pszTaskInformation);
        fwrite(pszTaskInformation, sizeof(char), stLength, pCom->pOut);
        fflush(pCom->pOut);

        /* Free the memory allocated for the string */
        free(pszTaskInformation);
    }
    else
    {
        fprintf(pCom->pOut, "Failed to get task information\r\n");
    }
    return CMD_OK;
}
/*****************************************************************************
End of function  cmdListTasks
******************************************************************************/

/******************************************************************************
Function Name: cmdVersion
Description:   Command to show the version
Arguments:     IN  iArgCount The number of arguments in the argument list
               IN  ppszArgument - The argument list
               IN  pCom - Pointer to the command object
Return value:  CMD_OK for success
******************************************************************************/
static int cmdVersion(int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    AVOID_UNUSED_WARNING;
    fprintf(pCom->pOut,
            WELCOME_MSG,
            "",
            "",
            WEB_ENGINE_VERSION,
            WEB_ENGINE_RELEASE,
            WEB_ENGINE_BUILD);
    return CMD_OK;
}
/******************************************************************************
End of function cmdVersion
******************************************************************************/

/******************************************************************************
Function Name: cmdHelp
Description:   Command to show the command help strings
Arguments:     IN  iArgCount The number of arguments in the argument list
               IN  ppszArgument - The argument list
               IN  pCom - Pointer to the command object
Return value:  CMD_OK for success
******************************************************************************/
static int cmdHelp(int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    int         iTable = 0;
    AVOID_UNUSED_WARNING;
    fprintf(pCom->pOut, "The commands available are:\r\n"
           "F1 or ? <CR> - print the command help strings\r\n"
           "F2 - brings up the last command line\r\n"
           "F3 - repeats the last command\r\n");

    while (iTable < pCom->iNumTables)
    {
        int         iCommand = 0;
        PCMDFNASS   pCommand = pCom->ppFn[iTable]->pCommandList;
        while (iCommand < pCom->ppFn[iTable]->dwNumberOfCommands)
        {
            /* Only print out if there is a string */
            if (pCommand->pszCommandDescription)
            {
                fprintf(pCom->pOut, "%s %s\r\n",
                       pCommand->pszCommand,
                       pCommand->pszCommandDescription);
            }
            pCommand++;
            iCommand++;
        }
        iTable++;
    }
    return CMD_OK;
}
/******************************************************************************
End of function cmdHelp
******************************************************************************/

/******************************************************************************
Function Name: cmdLastCommandLine
Description:   Command to bring up the last typed command
Arguments:     IN  iArgCount The number of arguments in the argument list
               IN  ppszArgument - The argument list
               IN  pCom - Pointer to the command object
Return value:  CMD_NO_PROMPT
******************************************************************************/
int cmdLastCommandLine(int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    AVOID_UNUSED_WARNING;
    conGetLastCommandLine(pCom);
    return CMD_NO_PROMPT;
}
/******************************************************************************
End of function cmdLastCommandLine
******************************************************************************/

/******************************************************************************
Function Name: cmdDoLastCommandLine
Description:   Command to bring up the last typed command
Arguments:     IN  iArgCount The number of arguments in the argument list
               IN  ppszArgument - The argument list
               IN  pCom - Pointer to the command object
Return value:  0 for success otherwise error code
******************************************************************************/
int cmdDoLastCommandLine(int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    AVOID_UNUSED_WARNING;
    return conDoLastCommandLine(pCom);
}
/******************************************************************************
End of function cmdDoLastCommandLine
******************************************************************************/

/******************************************************************************
Function Name: cmdSysInfo
Description:   Command to show the system information
Arguments:     IN  iArgCount The number of arguments in the argument list
               IN  ppszArgument - The argument list
               IN  pCom - Pointer to the command object
Return value:  0 for success otherwise error code
******************************************************************************/
int cmdSysInfo(int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    fprintf(pCom->pOut, "\r\n");
    cmdListTasks(iArgCount, ppszArgument, pCom);
    fprintf(pCom->pOut, "\r\n");
    cmdSystemUsage(iArgCount, ppszArgument, pCom);
    return CMD_OK;
}
/******************************************************************************
End of function cmdSysInfo
******************************************************************************/

/******************************************************************************
Function Name: cmdIdle
Description:   Command to do nothing
Arguments:     IN  iArgCount The number of arguments in the argument list
               IN  ppszArgument - The argument list
               IN  pCom - Pointer to the command object
Return value:  0 for success otherwise error code
******************************************************************************/
int cmdIdle(int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    AVOID_UNUSED_WARNING;
    fprintf(pCom->pOut, "\r\n");
    return CMD_OK;
}
/******************************************************************************
 End of function cmdIdle
 ******************************************************************************/

/* Table that associates command letters, function pointer and a little
   description of what the command does */
const CMDFNASS gpcmdCommand[] =
{
    {
        "date",
        cmdSetDate,
        "DD/MM/YYYY<CR> - Set the date"
    },

    {
        "time",
        cmdSetTime,
        "hh:mm:ss<CR> - Set the time"
    },

    {
        "mem",
        cmdReadMemory,
        "a l<CR> - Reads memory from address H'a length H'l"
    },

    {
        "sys",
        cmdSystemUsage,
        "<CR> - Shows the system resource usage information"
    },

    {
        "task",
        cmdListTasks,
        "n<CR> - List n tasks"
    },

    {
        "ver",
        cmdVersion,
        "<CR> - Show the application version"
    },

    {
        "?",
        cmdHelp,
        NULL
    },

    {
        "help",
        cmdHelp,
        NULL
    },

    /* Commands for doskey emulation of function keys */
                                    /* F1 - Print help strings */
   {
       "[11",
        cmdHelp,
        NULL
   },

   {
        "OP",
        cmdHelp,
        NULL
   },

   {
        "op",
        cmdHelp,
        NULL
   },

                                    /* F2 - get last command line */
   {
       "[12",
       cmdLastCommandLine,
        NULL
   },

   {
       "OQ",
       cmdLastCommandLine,
       NULL
   },

   {
       "oq",
       cmdLastCommandLine,
       NULL
   },

                                    /* F3 - do last command */
   {
       "[13",
       cmdDoLastCommandLine,
       NULL
   },

   {
       "OR",
       cmdDoLastCommandLine,
       NULL
   },

   {
       "or",
       cmdDoLastCommandLine,
       NULL
   },

                                    /* F4 - System info */
   {
       "[14",
       cmdSysInfo,
       NULL
   },

   {
       "OS",
       cmdSysInfo,
       NULL
   },

   {
       "os",
       cmdSysInfo,
       NULL
   },

   {
       "[A",                           /* Up arrow */
       cmdIdle,
       NULL
   },

   {
       "[B",                           /* Down arrow */
       cmdIdle,
       NULL
   },

   {
       "[C",                           /* Right arrow */
       cmdIdle,
       NULL
   },

   {
       "[D",                           /* Left arrow */
       cmdIdle,
       NULL
   }
};

/* Table that points to the above table and contains the number of entries */
const CMDTAB gcmdCommand =
{
    (PCMDFNASS)gpcmdCommand,
    (sizeof(gpcmdCommand)/sizeof(CMDFNASS))
};

/******************************************************************************
End  Of File
******************************************************************************/
