/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2011 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : mabMalloc.h
* Version      : 1.00
* Device(s)    : Renesas
* Tool-Chain   : Renesas
* OS           : None
* H/W Platform : SH2A
* Description  : Internal memory allocation functions
*******************************************************************************
* History      : DD.MM.YYYY Ver. Description
*              : 01.08.2009 1.00 First Release
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

#ifndef MABMALLOC_H_INCLUDED
#define MABMALLOC_H_INCLUDED

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include "FreeRTOS.h"
#include "task.h"
#include "r_typedefs.h"

/******************************************************************************
Macro definitions
******************************************************************************/
#define MAB_INVALID_TASK_HANDLE     (NULL)
#define MAB_START_ADDRESS(pMAB)     ((pMAB)->pbyHeapStart)
#define MAB_END_ADDRESS(pMAB)       ((pMAB)->pbyHeapEnd)
#define MAB_NOT_IN_USE(pMAB)        ((pMAB)->pbyHeap == (pMAB)->pbyHeapStart)
#define MAB_IS_IN_USE(pMAB)         ((pMAB)->pbyHeap != (pMAB)->pbyHeapStart)

/******************************************************************************
Typedef definitions
******************************************************************************/

/* Define the size that the memory allocated should be aligned to with this type.
 * It should be noted that if the heap start is not placed on this boundary
 * the the allocator's alignment will not work. Therefore please ensure that
 * the heap start addresses are aligned too! */
typedef struct
{
    unsigned char pbyAlign[32];
} mabalign_t;

typedef struct _MBL *PMBL;
typedef struct _MBL
{
    void        *pvTaskHandle;
    PMBL        pNext;
} MBL;

typedef union _UMBL
{
    MBL         memoryBlockList;
    mabalign_t  allignType;
} UMBL;

typedef struct _MAB
{
    /* Pointer to the top of the heap */
    uint8_t *   pbyHeap;

    /* Beginning of the heap memory */
    uint8_t *   pbyHeapStart;

    /* End of heap memory */
    uint8_t *   pbyHeapEnd;

    /* The Peak allocation since last reset */
    uint32_t    uiPeakAllocated;

    /* The current allocation since last reset */
    uint32_t    uiAllocated;
} MAB,
*PMAB;

typedef struct _MABINFO
{
    size_t  stSize;
    size_t  stOverhead;
    size_t  stTotalAllocated;
    size_t  stLargestAllocation;
    size_t  stTotalFree;
    size_t  stLargestFreeSegment;
    size_t  stPeakAllocated;

} MABINFO,
*PMABINFO;

/*****************************************************************************
Public Functions
******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
Function Name: mabMalloc
Description:   Function to allocate a block of memory
Arguments:     IN  pMAB - Pointer to the memory allocation block
               IN  pvTaskHandle - The ID of the task that requires the memory
               IN  stLength - The length of memory to allocate
Return value:  Pointer to the allocated memory or NULL if no memory free
******************************************************************************/

extern  void *mabMalloc(PMAB pMAB, void *pvTaskHandle, size_t stLength);

/******************************************************************************
Function Name: mabFree
Description:   Function to free a block of memory
Arguments:     IN  pMAB - Pointer to the memory allocation block
               IN  pvFree - Pointer to the start of a memory block to free
Return value:  none
******************************************************************************/

extern  void mabFree(PMAB pMAB, void *pvFree);

/*****************************************************************************
Function Name: mabFreeByTaskID
Description:   Function to free memory by the ID of the task that allocated it
Arguments:     IN  pMAB - Pointer to the memory allocation block
               IN  pvTaskHandle - The task ID
Return value:  The quantity of memory freed
*****************************************************************************/

extern  size_t mabFreeByTaskID(PMAB pMAB, void *pvTaskHandle);

/******************************************************************************
Function Name: mabRealloc
Description:   Function to change the size of the allocated memory
Arguments:     IN  pMAB - Pointer to the memory allocation block
               IN  pvReAlloc - Pointer to the memory to re-allocate
               IN  stLength - The new length of the block
Return value:  Pointer to the re-allocated memory or NULL if not enough free
******************************************************************************/

extern  void *mabRealloc(PMAB pMAB, void *pvReAlloc, size_t stLength);

#ifdef __cplusplus
}
#endif

#endif /* MABMALLOC_H_INCLUDED */

/******************************************************************************
End  Of File
******************************************************************************/
