/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2011 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : xvasprintf.h
* Version      : 1.00
* Description  : asprintf with out-of-memory checking.
*******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 01.02.2011 1.00    First Release
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

#ifndef _XVASPRINTF_H
#define _XVASPRINTF_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "r_typedefs.h"
#include "malloc.h"

/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Typedef definitions
******************************************************************************/

typedef struct _CAT
{
    /* Pointer to the allocated string */
    char        *pchBuffer;

    /* The length of the string */
    uint32_t    uiCount;

    /* The length of the allocated memory */
    uint32_t    uiLength;

    /* The size to allocate */
    uint32_t    uiChunk;
} CAT,
*PCAT;

/******************************************************************************
Macro definitions
******************************************************************************/

/* The CAT structure should have the all member set to 0 before using */
#define xcatcreate(pCat)            memset(pCat, 0, sizeof(CAT))
/* The string */
#define xcatstring(pCat)            ((pCat)->pchBuffer)
/* The string memory should be free'd when finished with */
#define xcatdestroy(pCat)           free((pCat)->pchBuffer);xcatcreate(pCat)

/******************************************************************************
Function Prototypes
******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
* Function Name: xasprintf
* Description  : Function to perform a formatted print output to a dynamically
                 allocated string
* Arguments    : IN  pszFormat - Pointer to a null terminated format string
*                I/O ... - The parameters
* Return Value : Pointer to the allocated string or NULL on error
******************************************************************************/

extern char *xasprintf(const char *pszFormat, ...);

/******************************************************************************
* Function Name: xcatprintf
* Description:   Function to perform concatenated print output to a dynamically
*                allocated string
* Parameters:    IN  pCat - Pointer to an initialise CAT object
*                IN  uiChunk - The size of block to allocate if required
*                IN  pszFormat - Pointer to a null terminated format string
*                I/O ... - The parameters
* Return value:  The total length of the string
******************************************************************************/

extern uint32_t xcatprintf(PCAT pCat, uint32_t uiChunk, char *pszFormat, ...);

#ifdef __cplusplus
}
#endif

#endif /* _XVASPRINTF_H */

/******************************************************************************
End  Of File
******************************************************************************/
