/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. This 
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : xvasprintf.c
* Version      : 1.00
* Description  : asprintf with out-of-memory checking.
*******************************************************************************
* History      : DD.MM.YYYY Version Description
*              : 04.04.2011 1.00    First Release
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include "xvasprintf.h"
#include "fmtOut.h"

/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/* Define the memory size that is allocated each time */
#define XAPRINTF_ALLOC_BLOCK_SIZE   128
#define XAPRINTF_ALLOC(x)           ((x < XAPRINTF_ALLOC_BLOCK_SIZE) ?\
                                    XAPRINTF_ALLOC_BLOCK_SIZE :\
                                    x)

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Function Prototypes
******************************************************************************/

static int32_t hPutChar(char chChar, void *pFileStream);

/******************************************************************************
Public Functions
******************************************************************************/

/******************************************************************************
* Function Name: xasprintf
* Description  : Function to perform a formatted print output to a dynamically
                 allocated string
* Arguments    : IN  pszFormat - Pointer to a null terminated format string
*                I/O ... - The parameters
* Return Value : Pointer to the allocated string or NULL on error
******************************************************************************/
char *xasprintf(const char *pszFormat, ...)
{
    CAT         printBuffer;
    va_list     ap;
    /* Initialise the print buffer */
    printBuffer.pchBuffer = malloc(XAPRINTF_ALLOC_BLOCK_SIZE);
    if (printBuffer.pchBuffer)
    {
        printBuffer.uiCount = 0;
        printBuffer.uiLength = XAPRINTF_ALLOC_BLOCK_SIZE;
        printBuffer.uiChunk = XAPRINTF_ALLOC_BLOCK_SIZE;
        /* Perform the formatted write */
        va_start(ap, pszFormat);
        fmtOut((char*)pszFormat, hPutChar, &printBuffer, ap);
        va_end(ap);
        if (printBuffer.pchBuffer)
        {
            /* Terminate the string */
            *(printBuffer.pchBuffer + printBuffer.uiCount) = '\0';
        }
    }
    return printBuffer.pchBuffer;
}
/******************************************************************************
End of function xasprintf
******************************************************************************/

/******************************************************************************
Function Name: xcatprintf
Description:   Function to perform concatenated print output to a dynamically
               allocated string
Parameters:    IN  pCat - Pointer to an initialise PCAT object
               IN  uiChunk - The size of block to allocate if required
               IN  pszFormat - Pointer to a null terminated format string
               I/O ... - The parameters
Return value:  The total length of the string
******************************************************************************/
uint32_t xcatprintf(PCAT pCat, uint32_t uiChunk, char *pszFormat, ...)
{
    va_list     ap;
    /* Check to see if this is a new object */
    if ((pCat->pchBuffer == NULL)
    &&  (pCat->uiLength == 0))
    {
        pCat->uiCount = 0;
        pCat->uiLength = uiChunk;
        pCat->uiChunk = uiChunk;
        pCat->pchBuffer = malloc((size_t)XAPRINTF_ALLOC(uiChunk));
    }
    if (pCat->pchBuffer)
    {
        pCat->uiChunk = uiChunk;
        /* Perform the formatted write */
        va_start(ap, pszFormat);
        fmtOut((char*)pszFormat, hPutChar, pCat, ap);
        va_end(ap);
        if (pCat->pchBuffer)
        {
            /* Terminate the string */
            *(pCat->pchBuffer + pCat->uiCount) = '\0';
        }
        else
        {
            xcatcreate(pCat);
        }
    }
    return pCat->uiCount;
}
/******************************************************************************
End of function  xcatprintf
******************************************************************************/

/******************************************************************************
Private Functions
******************************************************************************/

/******************************************************************************
* Function Name: hPutChar
* Description  : Function to put characters to the string
* Arguments    : IN chChar - The character to put
*                IN pFileStream - Pointer to the ANSI File Stream to put the
*                                 character to
* Return Value : 0 for success otherwise error code
******************************************************************************/
static int32_t hPutChar(char chChar, void *pFileStream)
{
    PCAT pPrintBuff = (PCAT)pFileStream;
    /* Write the data if the buffer becomes full */
    if (pPrintBuff->uiCount == pPrintBuff->uiLength)
    {
        void *pvBuffer = pPrintBuff->pchBuffer;
        pPrintBuff->uiLength += pPrintBuff->uiChunk;
        pPrintBuff->pchBuffer = realloc(pPrintBuff->pchBuffer,
                                        pPrintBuff->uiLength);
        if (!pPrintBuff->pchBuffer)
        {
            free(pvBuffer);
            return -1;
        }
    }
    /* Check that the buffer was allocted */
    if (pPrintBuff->pchBuffer)
    {
        /* Put the byte in the buffer */
        *(pPrintBuff->pchBuffer + pPrintBuff->uiCount++) = (char)chChar;
    }
    return 0;
}
/******************************************************************************
End of function hPutChar
******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
