/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. This 
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : drvLCDPMod.c
* Version      : 1.00
* Device(s)    : Renesas
* Tool-Chain   : GNU
* OS           : FreeRTOS
* H/W Platform : RZA1
* Description  : Timer device driver
*******************************************************************************
* History      : DD.MM.YYYY Version Description
*              : 04.02.2010 1.00    First Release
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include "r_typedefs.h"                   /* Default  type definition header */
#include "devdrv_common.h"                           /* Common Driver header */
#include "iodefine.h"                            /* I/O Register root header */
#include "rza_io_regrw.h"            /* Low level register read/write header */
#include "compiler_settings.h"   /* Interchangeable compiler specific header */
#include "FreeRTOS.h"
#include "timers.h"
#include "drvLCDPMod.h"
#include "trace.h"

/******************************************************************************
Macro definitions
******************************************************************************/

/* Comment this line out to turn ON module trace in this file */
//#undef _TRACE_ON_

#ifndef _TRACE_ON_
#undef TRACE
#define TRACE(x)
#endif

/*****************************************************************************
Function Prototypes
******************************************************************************/

static int8_t    LCDPModOpen(PSTREAM pStream);
static void   LCDPModClose(PSTREAM pStream);
static int8_t LCDPModControl(PSTREAM pStream, CTLCODE ctlCode, void *pCtlStruct);

/*****************************************************************************
Constant Data
******************************************************************************/

/* Define the driver function table for this */
const DEVICE gLCDPModDriver =
{
    "LCDPMod Device Driver",
    LCDPModOpen,
    LCDPModClose,
    nodevIO,
    nodevIO,
    LCDPModControl
};

/*****************************************************************************
Private Functions
******************************************************************************/

/******************************************************************************
Function Name: LCDPModOpen
Description:   Function to open the device driver
Arguments:     IN  pStream - Pointer to the file stream
Return value:  0 for success otherwise -1
******************************************************************************/
static int8_t LCDPModOpen(PSTREAM pStream)
{
    (void)pStream;

    rza_io_reg_write_16(&GPIO.PM1,   0, GPIO_PM1_PM14_SHIFT,    GPIO_PM1_PM14);

    return 0;
}
/******************************************************************************
End of function  LCDPModOpen
******************************************************************************/

/******************************************************************************
Function Name: LCDPModClose
Description:   Function to close the standard IN driver.
Arguments:     IN  pStream - Pointer to the file stream
Return value:  none
******************************************************************************/
static void LCDPModClose(PSTREAM pStream)
{
    (void)pStream;
}
/******************************************************************************
End of function  LCDPModClose
******************************************************************************/

/******************************************************************************
Function Name: LCDPModControl
Description:   Function to handle custom controls for the standard IN device
Arguments:     IN  pStream - Pointer to the file stream
               IN  ctlCode - The custom control code
               IN  pCtlStruct - Pointer to the custom control structure
Return value:  0 for success -1 on error
******************************************************************************/
static int8_t LCDPModControl(PSTREAM pStream, CTLCODE ctlCode, void *pCtlStruct)
{
    int8_t i_result = -1;

    /* Avoid unused parameter compiler warning */
    (void)pStream;

/*    if (pCtlStruct) */
    {
        switch (ctlCode)
        {
            case CTL_PMOD_REQ_DISPLAY_UPDATE:
            {
                i_result = 0;
                TRACE(("drvLCDPMod: CTL_PMOD_REQ_DISPLAY_UPDATE Request\r\n"));
                break;
            }

            default:
            {
                TRACE(("drvLCDPMod: Unknown control code\r\n"));
                break;
            }
        }
    }
    return i_result;
}
/******************************************************************************
End of function  LCDPModControl
******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
