/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. This 
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2011 Renesas Electronics Corporation. All rights reserved.
* Portions copyright (C) 2011  Swarm Systems Limited. All rights reserved.
*******************************************************************************
* File Name    : drvScif.h
* Version      : 1.0
* Device(s)    : Renesas
* Tool-Chain   : Renesas SH2A V9+
* OS           : None
* H/W Platform : SH2A
* Description  : Generic driver for the Serial Communications Interface with
*                FIFO
*******************************************************************************
* History      : DD.MM.YYYY Ver. Description
*              : 01.08.2009 1.00 First Release
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

#ifndef DDSCIF_H_INCLUDED
#define DDSCIF_H_INCLUDED

/******************************************************************************
User Includes
******************************************************************************/

#include "r_typedefs.h"
#include "control.h"
#include "devdrv_intc.h"
#include "r_cbuffer.h"
#include "IODefine.h"
#include "r_sysIf.h"
#include "r_event.h"

/******************************************************************************
Defines
******************************************************************************/

/******************************************************************************
Enumerated Types
******************************************************************************/

/******************************************************************************
Typedefs
******************************************************************************/

/* Define a type that points to the SCIF */
typedef volatile struct st_scif_n *PSCIF;
/* Define a pointer to a function to set the SCIF interrupt priority */
typedef void (*PSETIRQ)(_Bool bfEnable);

/* Define a structure to contain the SCIF configuration data */
typedef struct _SCIFCFG
{
    uint32_t   dwBaud;                      /* The desired baud rate */
    uint32_t   dwConfig;                    /* The SCIF line coding configuration*/
    uint32_t   dwPeripheralClockFrequency;  /* The frequency of the peripheral
                                               clock */
    PSETIRQ pSetIRQ;                        /* Pointer to a function to set the
                                               interrupt priority of the SCIF */
} SCIFCFG,
*PSCIFCFG;

/* Define the structure of data required by the driver */
typedef struct _DDSCIF
{
    PSCIF       pPORT;                      /* Pointer to the hardware */
    SCIFCFG     sciConfig;                  /* The SCI configuration */
    uint32_t    dwActualBaud;               /* The actual baud rate set */
    PEVENT      pevReceive;                 /* Set when data received */
    PCBUFF      pRxBuffer;                  /* Pointer to the receive software FIFO */
    PCBUFF      pTxBuffer;                  /* Pointer to the transmit software FIFO */
    SCIERR      errorCode;                  /* The last error code */
} DDSCIF,
*PDDSCIF;

/******************************************************************************
Public Functions
******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
Function Name: sciOpen
Description:   Function to configure the SCIF
Arguments:     I/O pDDSCIF - Pointer to the driver data
               IN  pSCIF - Pointer to the base address of the SCIF
               IN  pConfiguration - Pointer to the configuration data
               IN  pRxBuffer - Pointer to the Receive circular buffer
               IN  pTxBuffer - Pointer to the Transmit circular buffer
Return value:  0 for success otherwise error code
******************************************************************************/

extern  SCIERR sciOpen(PDDSCIF     pDDSCIF,
                       PSCIF       pSCIF,
                       PSCIFCFG    pConfiguration,
                       PCBUFF      pRxBuffer,
                       PCBUFF      pTxBuffer);

/******************************************************************************
Function Name: sciClose
Description:   Function to stop the SCIF
Arguments:     IN  pDDSCIF - Pointer to the driver data
Return value:  none
******************************************************************************/

extern  void sciClose(PDDSCIF pDDSCIF);

/******************************************************************************
Function Name: sciReConfigure
Description:   Function to re-configure the line-coding
Arguments:     IN  pDDSCIF - Pointer to the driver data
               IN  dwBaud - The desired baud rate
               IN  dwConfig - The desired configuration
Return value:  0 for success
******************************************************************************/

extern  int sciReConfigure(PDDSCIF pDDSCIF, uint32_t dwBaud, uint32_t dwConfig);

/******************************************************************************
Function Name: sciReadData
Description:   Function to start reading data from the SCIF
Arguments:     IN  pDDSCIF - Pointer to the driver data
               IN  pbyDest - Pointer to the destination memory
               IN  stLength - The length of data to read
Return value:  0 for success otherwise an error code
******************************************************************************/

extern  SCIERR sciReadData(PDDSCIF pDDSCIF, uint8_t *pbyDest, size_t stLength);

/******************************************************************************
Function Name: sciReadDataTimed
Description:   Function to start reading data from the SCIF for a period of time
Arguments:     IN  pDDSCIF - Pointer to the driver data
               IN  pbyDest - Pointer to the destination memory
               IN  stLength - The length of the data buffer
Return value:  The number of bytes read
******************************************************************************/

extern  int sciReadDataTimed(PDDSCIF pDDSCIF, uint8_t *pbyDest, size_t stLength);

/******************************************************************************
Function Name: sciWriteData
Description:   Function to start writing data to the SCIF
Arguments:     IN  pDDSCIF - Pointer to the driver data
               IN  pbySrc - Pointer to the source  memory
               IN  stLength - The length of data to write
Return value:  none
******************************************************************************/

extern  void sciWriteData(PDDSCIF pDDSCIF, uint8_t *pbySrc, size_t stLength);

/**********************************************************************************
Function Name: sciWaitTx
Description:   Function to wait for the TX software FIFO to become empty
Parameters:    IN  pDDSCIF - Pointer to the driver data
Return value:  none
**********************************************************************************/

extern  void sciWaitTx(PDDSCIF pDDSCIF);

/******************************************************************************
The interrupt service routines
******************************************************************************/

/******************************************************************************
Function Name: sciINTBreak
Description:   The break error interrupt handler
Arguments:     IN  pDDSCIF - Pointer to the driver data
Return value:  none
******************************************************************************/

extern  void sciINTBreak(PDDSCIF pDDSCIF);

/******************************************************************************
Function Name: sciINTError
Description:   The break error interrupt handler
Arguments:     IN  pDDSCIF - Pointer to the driver data
Return value:  none
******************************************************************************/

extern  void sciINTError(PDDSCIF pDDSCIF);

/******************************************************************************
Function Name: sciINTRxi
Description:   Receive data interrupt service routine handler
Arguments:     IN  pDDSCIF - Pointer to the driver data
Return value:  none
******************************************************************************/

extern  void sciINTRxi(PDDSCIF pDDSCIF);

/******************************************************************************
Function Name: sciINTTxi
Description:   Receive data interrupt service routine handler
Arguments:     IN  pDDSCIF - Pointer to the driver data
Return value:  none
******************************************************************************/

extern  void sciINTTxi(PDDSCIF pDDSCIF);

#endif /* DDSCIF_H_INCLUDED */

/******************************************************************************
End  Of File
******************************************************************************/
