/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. This 
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : drvTimer.c
* Version      : 1.00
* Device(s)    : Renesas
* Tool-Chain   : GNU
* OS           : FreeRTOS
* H/W Platform : RZA1
* Description  : Timer device driver
*******************************************************************************
* History      : DD.MM.YYYY Version Description

*              : 04.02.2010 1.00    First Release
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include "r_typedefs.h"
#include "iodefine.h"
#include "FreeRTOS.h"
#include "timers.h"
#include "drvTimer.h"
#include "trace.h"
#include "hwwdt.h"


/******************************************************************************
Macro definitions
******************************************************************************/

/* Comment this line out to turn ON module trace in this file */
#undef _TRACE_ON_

#ifndef _TRACE_ON_
#undef TRACE
#define TRACE(x)
#endif

/*****************************************************************************
Function Prototypes
******************************************************************************/

static int tmrOpen(PSTREAM pStream);
static void tmrClose(PSTREAM pStream);
static int tmrControl(PSTREAM pStream, CTLCODE ctlCode, void *pCtlStruct);

/*****************************************************************************
Constant Data
******************************************************************************/

/* Define the driver function table for this */
const DEVICE gTimerDriver =
{
    "Timer Device Driver",
    tmrOpen,
    tmrClose,
    nodevIO,
    nodevIO,
    tmrControl
};

/*****************************************************************************
Private Functions
******************************************************************************/

/******************************************************************************
Function Name: tmrOpen
Description:   Function to open the system timer device driver
Arguments:     IN  pStream - Pointer to the file stream
Return value:  0 for success otherwise -1
******************************************************************************/
static int tmrOpen(PSTREAM pStream)
{
    /* No open function is required as this is done for the initialisation
       of FreeRTOS */
    (void)pStream;
    return 0;
}
/******************************************************************************
End of function  tmrOpen
******************************************************************************/

/******************************************************************************
Function Name: tmrClose
Description:   Function to close the standard IN driver. All open devices
               connected to the tmr are closed.
Arguments:     IN  pStream - Pointer to the file stream
Return value:  none
******************************************************************************/
static void tmrClose(PSTREAM pStream)
{
    (void)pStream;
}
/******************************************************************************
End of function  tmrClose
******************************************************************************/

/******************************************************************************
Function Name: tmrControl
Description:   Function to handle custom controls for the standard IN device
Arguments:     IN  pStream - Pointer to the file stream
               IN  ctlCode - The custom control code
               IN  pCtlStruct - Pointer to the custom control structure
Return value:  0 for success -1 on error
******************************************************************************/
static int tmrControl(PSTREAM pStream, CTLCODE ctlCode, void *pCtlStruct)
{
    int iResult = -1;
    /* Avoid unused parameter compiler warning */
    (void)pStream;
    if (pCtlStruct)
    {
        switch (ctlCode)
        {
            case CTL_GET_TIME_STAMP:
            {
                PTMSTMP     pPerfTimer = (PTMSTMP)pCtlStruct;
                uint32_t    ulTimeNow = OSTM1.OSTMnCNT;
                pPerfTimer->ulMilisecond = ulTimeNow / (portTICK_CNT_FREQ_HZ / 1000);
                pPerfTimer->usSubMilisecond = (ulTimeNow % (portTICK_CNT_FREQ_HZ / 1000));
                iResult = 0;
                break;
            }
            case CTL_WDT_RESET:
            {
                wdtHardwareReset();
                break;
            }
            default:
            {
                TRACE(("drvTimer: Unknown control code\r\n"));
                break;
            }
        }
    }
    return iResult;
}
/******************************************************************************
End of function  tmrControl
******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
