/******************************************************************************
  Copyright (C) 2011 Renesas Electronics Corporation, All Rights Reserved.
*******************************************************************************
* File Name    : hwRIIC3.h
* Version      : 1.00
* Description  : Hardware interface for RIIC3
******************************************************************************
* History : DD.MM.YYYY Version Description
*         : dd.mm.yyyy 1.00    First Release
******************************************************************************/

#ifndef HWRIIC3_H_INCLUDED
#define HWRIIC3_H_INCLUDED

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
System includes / User includes
******************************************************************************/
#include "r_typedefs.h"

/******************************************************************************
Public Functions
******************************************************************************/


#define PE_TFT_CS_MSK      (0x04)
#define PE_MUX_MSK         (0x01)

#define ENABLE_TFT_CS_CMD  (0x00)
#define DISABLE_TFT_CS_CMD (0x04)
#define ENABLE_MUX_CMD     (0x00)
#define DISABLE_MUX_CMD    (0x01)


/******************************************************************************
* Function Name: r_RIIC3_Open
* Description  : Function to open RIIC channel 3
* Arguments    : none
* Return Value : 0 for success or -1 on error
******************************************************************************/

extern  int32_t r_RIIC3_Open(void);

/******************************************************************************
* Function Name: r_RIIC3_Close
* Description  : Function to close RIIC channel 3
* Arguments    : none
* Return Value : none
******************************************************************************/

extern  void r_RIIC3_Close(void);

/******************************************************************************
* Function Name: r_RIIC3_UpdateState_PE1
* Description  : Function to modify output state of Port Expander 1
*                on RIIC channel 3
* Arguments    : IN  mask - which pin(s) to modify
*                IN  state - new state for pin(s)
* Return Value : none
******************************************************************************/

extern void R_RIIC3_UpdateState_PE1(uint8_t mask, uint8_t state);

/******************************************************************************
* Function Name: r_RIIC3_UpdateState_PE2
* Description  : Function to modify output state of Port Expander 2
*                on RIIC channel 3
* Arguments    : IN  mask - which pin(s) to modify
*                IN  state - new state for pin(s)
* Return Value : none
******************************************************************************/

extern void R_RIIC3_UpdateState_PE2(uint8_t mask, uint8_t state);

/******************************************************************************
* Function Name: r_RIIC3_TxStart
* Description  : Function to transmit the start condition
* Arguments    : none
* Return Value : 0 for success or -1 on error
******************************************************************************/

extern  int32_t r_RIIC3_TxStart(void);

/******************************************************************************
* Function Name: r_RIIC3_TxData
* Description  : Function to transmit data
* Arguments    : IN  pvData - Pointer to the data to transmit
*                IN  stLength - The length of data to transmit
* Return Value : 0 for success or -1 on error
******************************************************************************/

extern  int32_t r_RIIC3_TxData(const void *pvData, size_t stLength);

/******************************************************************************
* Function Name: r_RIIC3_RxData
* Description  : Function to receive data
* Arguments    : IN  pvData - Pointer to the data buffer
*                IN  stLength - The length of data to receive
* Return Value : 0 for success or -1 on error
******************************************************************************/

extern  int32_t r_RIIC3_RxData(void *pvData, size_t stLength);

/******************************************************************************
* Function Name: r_RIIC3TxStop
* Description  : Function to send the stop condition
* Arguments    : none
* Return Value : 0 for success or -1 on error
******************************************************************************/

extern  int32_t r_RIIC3_TxStop(void);


/******************************************************************************
* Function Name: r_RIIC3_Initialise
* Description  : Function to initialise i2c channel with default port config
* Arguments    : none
* Return Value : none
******************************************************************************/

extern void r_RIIC3_Initialise(void);

#ifdef __cplusplus
}
#endif

#endif                              /* HWRIIC3_H_INCLUDED */

/*****************************************************************************
End  Of File
******************************************************************************/
