/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. This 
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : hwLed.c
* Version      : 1.00
* Device(s)    : Renesas
* Tool-Chain   : Renesas SH2A V9+
* OS           : None
* H/W Platform : RSK+SH7269
* Description  : LED port test
*******************************************************************************
* History      : DD.MM.YYYY Ver. Description
*              : 05.08.2010 1.00 First Release
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "hwLCDPMod.h"
#include "control.h"
#include "r_task.h"
#include "rspi.h"
#include "r_lcd_pmod.h"

/*******************************************************************************
Defines
*******************************************************************************/
#define BMP_OFFSET (10u)

static void task_lcd_pmod(void);

/******************************************************************************
Exported global variables and functions
******************************************************************************/
int r_lcd_gipPmod = -1;
uint16_t my_image[2][128*128*3];

extern uint8_t g_acLogo[];

/******************************************************************************
* Function Name: task_lcd_pmod
* Description  : task to manage LCD_PMOD Display
* Arguments    : none
* Return Value : none
******************************************************************************/
static void task_lcd_pmod(void)
{
    const int32_t i_sleep_count = 1000UL;

    R_LCD_DisplayBMPImage (&g_acLogo[0], 128, 24, 0, 104);
    R_LCD_UpdateDisplay();
    taskSleep(i_sleep_count);
    taskExit();

}

/******************************************************************************
End of function  task_lcd_pmod
******************************************************************************/

/*******************************************************************************
* Function Name : lcd_pmod_init
* Description   : Initialises the PMod display
*                 Set the font colour to white.
* Argument      : none
* Return value  : none
*******************************************************************************/
void lcd_pmod_init (void)
{
    uint32_t taskid;

    R_LCD_ResourceLockPMOD();

    /* Open the LED interface */
    r_lcd_gipPmod = open(DEVICE_INDENTIFIER "lcdpmod", 0);

    /* Initialise SPI Used by PMOD */
    R_SPI_Init();

    /* initialise Standard PMOD display */
    R_LCD_Init();

    R_LCD_DisplayOff();
    R_LCD_DisplayBMPImage (&g_acLogo[0], 128, 24, 0, 104);
    R_LCD_DisplayStringAtLine(1, (uint8_t *)" RSK+RZA1H");
    R_LCD_DisplayStringAtLine(3, (uint8_t *)" FreeRTOS Sample");
    R_LCD_UpdateDisplay();
    R_LCD_DisplayOn();

    taskid = taskCreate((PTASKFN)task_lcd_pmod,
                       NULL,
                       configSMALL_STACK_SIZE,
                       TC_DEFAULT_HEAP,
                       "PMOD init",
                       0);
    taskSleep(200);
    R_LCD_ResourceUnlockPMOD();

    taskDestroy(taskid);
}

/*******************************************************************************
* End of function lcd_pmod_init
*******************************************************************************/

/*******************************************************************************
* Function Name : lcd_pmod_display_image
* Description   : This function puts a 24-bit image on the 16-bit display,
*                 colour reduction is embedded in display routine.
* Argument      : uint8_t  *image       - 24 bit RGB888 format RAW data no
*                                         header
*                 uint16_t image_width  - Target image width
*                 uint16_t image_height - Target image height
*                 uint16_t loc_x        - Image horizontal position on display
*                 uint16_t loc_y        - Image vertical position on display
* Return value  : none
*******************************************************************************/
void lcd_pmod_display_image (uint8_t *image, uint8_t image_width,
                            uint8_t image_height, uint8_t loc_x, uint8_t loc_y)
{
    uint16_t count = (image_height * (image_width*3));
    uint8_t *pmyd0 =  ((uint8_t*)&my_image[0]);
    uint8_t *pmyd1 =  ((uint8_t*)&my_image[1]);

    memcpy(pmyd1,pmyd0,count);
    memcpy(pmyd0,(image + image[BMP_OFFSET]),count);

    control(r_lcd_gipPmod, CTL_PMOD_REQ_DISPLAY_UPDATE, NULL);
}

/*******************************************************************************
* End of function lcd_pmod_display_image
*******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
