/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : hwLed.c
* Version      : 1.00
* Device(s)    : Renesas
* Tool-Chain   : Renesas SH2A V9+
* OS           : None
* H/W Platform : RSK+SH7269
* Description  : LED port test
*******************************************************************************
* History      : DD.MM.YYYY Ver. Description
*              : 05.08.2010 1.00 First Release
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "hwLed.h"
#include "control.h"

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/
extern int g_iled;

/******************************************************************************
Function Name: led_on
Description:   LED specified for iLedNum is turned on.
Arguments:     IN iLedNum : LED number
Return value:  none
******************************************************************************/
void led_on(int32_t iLedNum)
{
    control(g_iled, CTL_SET_LED_ON, &iLedNum);
}
/******************************************************************************
End of function  led_on
******************************************************************************/

/******************************************************************************
Function Name: led_toggle
Description:   LED specified for iLedNum is toggled.
Parameters:    IN iLedNum : LED number
Return value:  none
******************************************************************************/
void led_toggle(int32_t iLedNum)
{
    control(g_iled, CTL_SET_LED_TOGGLE, &iLedNum);
}
/******************************************************************************
End of function  led_toggle
******************************************************************************/

/******************************************************************************
Function Name: led_off
Description:   LED specified for iLedNum is tuned off.
Arguments:     IN iLedNum : LED number
Return value:  none
******************************************************************************/
void led_off(int32_t iLedNum)
{
    control(g_iled, CTL_SET_LED_OFF, &iLedNum);
}
/******************************************************************************
End of function  led_off
******************************************************************************/

/******************************************************************************
Function Name: led_status
Description:   LED status specified for "iLedNum".
Arguments:     IN iLedNum : LED number
Return value:  0 led on
               1 led off
******************************************************************************/
int32_t led_status(int32_t iLedNum)
{
    int32_t iLedMap = 0;
    /* Get the map, LEDs are logic 1 when on */
    control(g_iled, CTL_GET_LED_MAP, &iLedMap);
    /* Check to see if this LED is on */
    if (iLedNum & iLedMap)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}
/******************************************************************************
End of function  led_off
******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
