/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. This 
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2013 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : r_sysif.h
* Version      : 1.02
* Description  : The system interface functions to abstract the WebEngine
*                from any particuar operating system. 
******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 04.02.2010 1.00    First Release
*         : 10.06.2010 1.01    Updated type definitions
*         : 10.11.2010 1.02    Updated get signal state
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

#ifndef SYSIF_H_INCLUDED
#define SYSIF_H_INCLUDED

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/


/******************************************************************************
Typedef definitions
******************************************************************************/

typedef enum _SIGST
{
    SYSIF_SIGNAL_RESET = 0,
    SYSIF_SIGNAL_SET,
    SYSIF_SIGNAL_ERROR
} SIGST;

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/*****************************************************************************
* Function Name: sysYield
* Description  : Function to yield to other tasks or threads
* Arguments    : none
* Return Value : none
******************************************************************************/

extern  void sysYield(void);

/*****************************************************************************
* Function Name: sysLock
* Description  : Function to lock a critical section
* Arguments    : IN  pvLockObj - Pointer to an operating system locking object
*                                or NULL if not requred
* Return Value : The lock value or -1 on error
******************************************************************************/

extern  int sysLock(void *pvLockObj);

/*****************************************************************************
* Function Name: sysUnlock
* Description  : Function to unlock a critical section
* Arguments    : IN  pvLockObj - Pointer to an operating system locking object
*                                or NULL if not requred
*                IN  iUnlock - The value returned from sysLock function
* Return Value : none
******************************************************************************/

extern  void sysUnlock(void *pvLockObj, int iUnlock);

/*****************************************************************************
Function Name: sysWaitAccess
Description:   Function to allow the same task to enter a critical section and
               other tasks wait
Arguments:     none
Return value:  none
*****************************************************************************/

extern  void sysWaitAccess(void);

/*****************************************************************************
Function Name: sysReleaseAccess
Description:   Function to allow other tasks to access a re-entrant critial
               section
Arguments:     none
Return value:  none
*****************************************************************************/

extern  void sysReleaseAccess(void);

#ifdef __cplusplus
}
#endif

#endif                              /* SYSIF_H_INCLUDED */

/******************************************************************************
End  Of File
******************************************************************************/
