/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. This 
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : r_timer.h
* Version      : 1.0
* Device(s)    : Renesas
* Tool-Chain   : GNU
* OS           : FreeRTOS
* H/W Platform : RZA1
* Description  : Timer functions
*******************************************************************************
* History      : DD.MM.YYYY Version Description
*              : 05.08.2010 1.00    First Release
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

#ifndef TIMER_H_INCLUDED
#define TIMER_H_INCLUDED

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include "r_task.h"
#include "control.h"

/******************************************************************************
Function Prototypes
******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/*****************************************************************************
Function Name: timerStartCallBack
Description:   Function to start a call-back timer
Description:   Function to start a call-back timer
Arguments:     IN  pTimer - Pointer to the call-back timer object
               IN  pTimerFunction - Pointer to the timer function to call
               IN  pvParameter - Pointer to the parameter to pass the function
               IN  ulTime_mS - The time in mS
               IN  Type - The type
Return value:  0 for success or -1 on error
*****************************************************************************/

extern  int timerStartCallBack(PCBTMR  pTimer,
                               PCFNTMR pTimerFunction,
                               void    *pvParameter,
                               uint32_t ulTime_mS,
                               TMRTYPE Type);

/*****************************************************************************
Function Name: timerStopCallBack
Description:   Function to stop a call-back timer
Arguments:     IN  pTimer - Pointer to the call-back timer structure
Return value:  0 for success
*****************************************************************************/

extern  int timerStopCallBack(PCBTMR pTimer);

/*****************************************************************************
Function Name: timerStartEvent
Description:   Function to start an event timer
Arguments:     IN  tmrType - The type of timer to start, SINGLE_SHOT_TIMER or
                             PERIODIC_TIMER
               IN  dwTime_mS - The time in mS
Return value:  Pointer to the event or NULL on error
*****************************************************************************/

extern  PEVENT timerStartEvent(TMRTYPE tmrType, uint32_t dwTime_mS);

/*****************************************************************************
Function Name: timerStopEvent
Description:   Function to stop an event timer
Arguments:     IN  pEvent - Pointer to the event timer to stop
Return value:  none
*****************************************************************************/

extern  void timerStopEvent(PEVENT pEvent);

/*****************************************************************************
Function Name: timerStartMeasurement
Description:   Function to start a measurement timer (low resolution)
Arguments:     OUT - pTimeStamp - Pointer to the destination time stamp
Return value:  none
*****************************************************************************/

extern  void timerStartMeasurement(PTMSTMP pTimeStamp);

/*****************************************************************************
Function Name: timerStopMeasurement
Description:   Function to stop a measurement timer and return the elapsed time
Arguments:     IN  pTimeStamp - Pointer to the starting time stamp
Return value:  The elapsed time in seconds
*****************************************************************************/

extern  float timerStopMeasurement(PTMSTMP pTimeStamp);

/*****************************************************************************
Function Name: timerUsage
Description:   Function to get the timer usage information
Arguments:     OUT pstInUse - Pointer to the numebr of timers in use
               OUT pstTotal - Pointer to the total number of timers available
Return value:  none
*****************************************************************************/

extern  void timerUsage(size_t *pstInUse, size_t *pstTotal);

#ifdef __cplusplus
}
#endif

#endif /* TIMER_H_INCLUDED */

/******************************************************************************
End  Of File
******************************************************************************/
