OUTPUT_FORMAT("elf32-littlearm", "elf32-bigarm", "elf32-littlearm")
OUTPUT_ARCH(arm)
ENTRY(start)

/* Base Address RAM Memory Table 10-Mbyte on-chip RAM */  
/* Total RAM available 10112K */
MEMORY {
    RAM0L (rwx)   : ORIGIN = 0x60020000, LENGTH = 0x004E0000  /* Page 0 Lower bank (1024kB) H'20000000 to H'200FFFFF */
    SDRAM (rwx)   : ORIGIN = 0x48000000, LENGTH = 0x02000000  /* SDRAM 32MB attached to CS2 H'08000000 to H'09FFFFFF */
    VRAM  (rw)    : ORIGIN = 0x60500000, LENGTH = 0x00500000  
}

MEMORY_MAP              = 0x60000000;

ALIGN04                 = 0x04;       /* Alignment 4 bytes                  */
ALIGN08                 = 0x08;       /* Alignment 8 bytes                  */
ALIGN4000               = 0x4000;     /* Alignment 16384 (16K) bytes        */

PROGRAM_STACK_SIZE      = 0x2000;     /* Application stack                  */
IRQ_STACK_SIZE          = 0x2000;     /* IRQ mode stack                     */
FIQ_STACK_SIZE          = 0x1000;     /* FRQ mode stack                     */
SVC_STACK_SIZE          = 0x2000;     /* SVC mode stack                     */
ABT_STACK_SIZE          = 0x1000;     /* ABT mode stack                     */
TTB_SIZE                = 0x8000;     /* Level-1 Translation Table for MMU  */
NL_HEAP_SIZE            = 0xC800;     /* Newlib heap size 50k Bytes         */
BUF_HEAP_SIZE           = 0x1000;

EXEC_BASE               = (MEMORY_MAP + 0x40000);  /* VECTOR_TABLE located here        */
STACK_BASE              = (MEMORY_MAP + 0xF0000);  /* ARM STACKS located here          */

VRAM_USER_SECTION       = (MEMORY_MAP + 0x600000); /* Graphics Screen Buffers          */
WORK_USER_SECTION1      = (MEMORY_MAP + 0x900000); /* Graphics RGA Buffers             */

SDRAM_START_ADDRESS     = 0x48000000; /* The start address of the external SDRAM */
SDRAM_SIZE              = 0x02000000; /* Size of the external SDRAM */

SECTIONS
{
    .reset EXEC_BASE :
    {
        execute = .;
        *start.o (.text)
        . = ALIGN(ALIGN04);
        *(.text.startup)
        address_end_reset = .;
    } > RAM0L 


    .data :
    {
        _sdata = .;
        *(.data)
        _data = .;
        _edata = .;
    } > RAM0L 
 
    .DATA_RESET :
    {
        _sDATA_RESET = .;
        *(.DATA_RESET)
        _eDATA_RESET = .;
    } > RAM0L 

    .program_stack STACK_BASE :
    {
        .  = ALIGN(ALIGN08);
        program_stack_start = .;
        . += PROGRAM_STACK_SIZE;
        .  = ALIGN(ALIGN08);
        program_stack_end = .; 
    } >RAM0L 
    
    .irq_stack :
    {
        .  = ALIGN(ALIGN08);
        irq_stack_start = .;
        . += IRQ_STACK_SIZE;
        .  = ALIGN(ALIGN08);
        irq_stack_end = .;
    } >RAM0L 
        
    .fiq_stack :
    {
        .  = ALIGN(ALIGN08);
        fiq_stack_start = .;
        . += FIQ_STACK_SIZE;
        .  = ALIGN(ALIGN08);
        fiq_stack_end = .;
    } >RAM0L 
        
    .svc_stack :
    {
        .  = ALIGN(ALIGN08);
        PROVIDE(svc_stack_start = .);        
        . += SVC_STACK_SIZE;
        .  = ALIGN(ALIGN08);
         svc_stack_end = .;
    } >RAM0L 
        
    .abt_stack :
    {
        .  = ALIGN(ALIGN08);
        abt_stack_start = .;
        . += ABT_STACK_SIZE;
        .  = ALIGN(ALIGN08);
        abt_stack_end = .;
    } >RAM0L 

    .ttb_mmu1 :
    {
        . = ALIGN(ALIGN4000);
        ttb_mmu1_base = .;
        . += TTB_SIZE;
        . = ALIGN(ALIGN08);
        ttb_mmu1_end = .;
    } >RAM0L 

    .rodata : 
    {
        _srodata = .;
        *(.rodata)
        *(.rodata.*)
        .  = ALIGN(ALIGN08);
        _erodata = .;
    } > RAM0L 


    .BSS_RESET : 
    {
        .  = ALIGN(ALIGN08);
        PROVIDE (__start_BSS_RESET = .);
        *(.BSS_RESET)
        PROVIDE (__stop_BSS_RESET = .);
        .  = ALIGN(ALIGN08);
    } > RAM0L 
        
    .CODE_RESET :
    {
        _sCODE_RESET = .;
        *(.CODE_RESET)
        _eCODE_RESET = .;
    } > RAM0L 

    .text2 :
    {
        *(.text)
    } > RAM0L 

    .bss :
    {
         _bss = .;
         PROVIDE(__bss_start__ = .);         
         *(.bss)
         *(.bss.**)
         *(COMMON)
         PROVIDE(__bss_end__ = .);
         _ebss = .;
         _end = .;
         PROVIDE(end = .);
    } > RAM0L 

    /* Heap for use by Newlib via the sbrk allocation */
    .NL_HEAP (NOLOAD)  :
    {
        . = ALIGN(ALIGN08);
        _nl_heap_start = .; 
        .   += NL_HEAP_SIZE;
        _nl_heap_end = .;
    } > RAM0L 
       
    .RAM1L_HEAP (NOLOAD)  :
    {
        . = ALIGN(ALIGN08);
        /* Something is missing! Need to add "space" from the end of 
           "used" memory. The linker must be putting stuff in this
           space but currently I don't know what or why */
        .   += BUF_HEAP_SIZE;
        _sram_heap_start = .; 
    } > RAM0L 


    /* NOLOAD directs linker NOT to fill VRAMx_SECTION with 0.          */
    /* Usage of NOLOAD increases speed of linker and download to target */
    /* These sections must be at uncached */
    .VRAM_SECTION  VRAM_USER_SECTION (NOLOAD) : 
    {
        _sram_heap_end = .;
    }  
    .WORK_SECTION1 WORK_USER_SECTION1 (NOLOAD) : {} 
    VRAM_EX  (NOLOAD) : {} > SDRAM

    .sdram_heap :
    {
        _sdram_heap_start = .;
        .  += SDRAM_SIZE;
        _sdram_heap_end = .;
    } >SDRAM        
}
