/******************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only
 * intended for use with Renesas products. No other uses are authorized. This
 * software is owned by Renesas Electronics Corporation and is protected under
 * all applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
 * LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
 * ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
 * FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
 * ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software
 * and to discontinue the availability of this software. By using this software,
 * you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 * Copyright (C) 2014 Renesas Electronics Corporation. All rights reserved.
 *****************************************************************************/
/******************************************************************************
 * File Name    : resetprg.c
 * Device(s)    : __#device#__
 * Tool-Chain   : __#tool_chain#__
 * OS           : __#OS_Name#__
 * H/W Platform : __#platform#__
 * $Rev         : 0 $
 * $Date        : 0#$
 * Description  : Sample Program - C library entry point
 *              : Variants of this file must be created for each compiler
 *****************************************************************************/
/******************************************************************************
 * History     : DD.MM.YYYY Version Description
 *             : 18.06.2013 1.00
 *             : __#dd#__.__#mm#__.__#yyyy#__ __#ver_maj#__.__#ver_min#__ First Release
 *****************************************************************************/

/******************************************************************************
 WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
 OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
 SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
 ******************************************************************************/

/******************************************************************************
 Includes   <System Includes> , "Project Includes"
 *****************************************************************************/
#include "r_typedefs.h"
#include "devdrv_common.h"      /* Common Driver Header */
#include "devdrv_intc.h"        /* INTC Driver Header   */
#include "r_sys_resetprg.h"
#include "ramTest.h"
#include "iodefine.h"
#include "FreeRTOS.h"
#include "Task.h"
#include "taskPriority.h"
#include "pl310_l2cc.h"
#include "compiler_settings.h"
#include "r_sys_hardwaresetup.h"
#include "r_sys_initiolib.h"
#include "r_sys_asmfunctions.h"
#include "irq.h"

/******************************************************************************
 Macro definitions
 *****************************************************************************/
#if defined(__thumb2__) || (defined(__thumb__) && defined(__ARM_ARCH_6M__))
# define THUMB_V7_V6M
#endif

/* Defined if this target supports the BLX Rm instruction.  */
#if  !defined(__ARM_ARCH_2__) && !defined(__ARM_ARCH_3__) && !defined(__ARM_ARCH_3M__)    && !defined(__ARM_ARCH_4__)    && !defined(__ARM_ARCH_4T__)
# define HAVE_CALL_INDIRECT
#endif

#ifdef HAVE_INITFINI_ARRAY
/* Note: Definition not in parenthesis due to compiler directive. */
/* Note: Macro Name does not meet coding standard intentionally. */
#define _init    __libc_init_array
#define _fini    __libc_fini_array
#endif

/******************************************************************************
 Typedef definitions
 *****************************************************************************/

/******************************************************************************
 Exported global variables (to be accessed by other files)
 *****************************************************************************/

/******************************************************************************
 Private global variables and functions
 *****************************************************************************/

static void main_task (void);

/******************************************************************************
 Imported global variables and functions (from other files)
 *****************************************************************************/

/******************************************************************************
 * Function Name: R_SYS_ResetPrg
 * Description  : The power on reset function
 * Arguments    : none
 * Return Value : none
 *****************************************************************************/
void R_SYS_ResetPrg (void)
{
    /* Setup the hardware */
    R_SYS_HardwareSetup();

    /* INTC setting */
    R_INTC_Init();


    /* Test the RAM */
    if ((ramSizeTest(SDRAM_START, SDRAM_SIZE) != SDRAM_SIZE)
            || (ramFillTest(SDRAM_START, (SDRAM_SIZE * sizeof(bus_width_t)))))
    {

        /* Flash LED1 forever */
        while (1)
        {

            /* Switch the LED on */
            PORT7.Pn.BIT.Pn1 = 0;
            {
                int32_t loop_count = 0x000100000;
                while (loop_count--)
                {
                    /* Wait */;
                }
            }

            /* Switch the LED off */
            PORT7.Pn.BIT.Pn1 = 1;
            {
                int32_t loop_count = 0x000100000;
                while (loop_count--)
                {
                    /* Wait */;
                }
            }
        }
    }

#ifdef R_FREERTOS_CONFIG_L2CC
    /* Initial setting of the level 2 cache */
    init_l2cc();
#endif

    /* Initial setting of the level 1 cache */
    R_SYS_L1CacheInit();

    /* Vector base address setting */
    R_SYS_VbarInit();

    __enable_irq();
    __enable_fiq();

    /* Create the main task */
    xTaskCreate((pdTASK_CODE )main_task, (signed char * )"Main",
            configSMALL_STACK_SIZE,
            NULL, TASK_MAIN_TASK_PRI, NULL);

    /* Start the FreeRTOS scheduler */
    vTaskStartScheduler();

    while (1)
    {
        /* This point should never be reached */
        ;
    }
}
/******************************************************************************
 End of function  R_SYS_ResetPrg
 *****************************************************************************/

/******************************************************************************
 * Function Name: main_task
 * Description  : The main task
 * Arguments    : none
 * Return Value : none
 *****************************************************************************/
static void main_task (void)
{

    /* Initialise the IO library - This must be done after the scheduler has
     * been started as it uses objects provided by FreeRTOS */
    R_SYS_InitIolib();

    /* Call the main task */
    main();

    while (1)
    {
        /* Stops program from running off */
        ;
    }
}
/******************************************************************************
 End of function  main_task
 *****************************************************************************/
