/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name     : ascii.c
* Device(s)     : RZ/A1H (R7S721001)
* Tool-Chain    : GNUARM-RZv14.01-EABI
* H/W Platform  : RSK+RZA1H CPU Board
* Description   : ascii module .h file
*******************************************************************************/
/*******************************************************************************
* History       : DD.MM.YYYY Version Description
*               : 19.06.15 Ver.1.00 Acting as a collector for seperate module
*               : Sep.02,2010 Ver.0.01.00 Preliminary version issued
*               : Oct.06.2010 Ver.0.02.00 VDC4.GR1_AB1 modified
*               : Oct.07.2010 Ver.0.03.00 VDC4.GR1_AB1 type definition modified
*               : Oct.19.2010 Ver.0.04.00 MMC.CE_DMA_MODE added
*                                         MMC.CE_BOOT deleted
*               : Nov.09.2010 Ver.0.05.00 VDC4.GR3_CLUT_INT.GR3_LINE added
*               : Jan.28.2011 Ver.0.06.00 DVDEC.ADDCR->ADCCR1 changed
*                                         DVDEC.ADCCR1.AGCMODEXA->AGCMODE changed
*                                         DVDEC.INSCR deleted
*                                         DVDEC.AGCCR2.AGCMAXGAIN deleted
*                                         DVDEC.AGCCR2.VIDEOGAIN deleted
*                                         DVDEC.AGCCR2.VIDEOGAIN deleted
*                                         DVDEC.CROMASR2.NCOMODE deleted
*                                         DVDEC.DCPSR3~5 deleted
*                                         DVDEC.YCSCR1 deleted
*                                         DVDEC.YCSCR3~7,9,11 added
*                                         DVDEC.YCSCR8.HFIL_TAP_SEL added
*                                         DVDEC.YCSCR12.DET2_MIX_C added
*                                         DVDEC.YCSCR12.DET2_MIX_Y added
*                                         DVDEC.DCPCR9.CLP_FIL_SEL deleted
*                                         DVDEC.DCPCR10~13 deleted
*                                         DVDEC.PGA_UPDATE added
*                                         DVDEC.PGACR added
*                                         DVDEC.ADCCR2 added
*                                         module SPIBSC added
*               : Feb.23.2011 Ver.0.07.00 CPG.STBCR7.MSTP75 added
*               : Feb.28.2011 Ver.0.08.00 PORT.PBCR5 modified
*                                         PORT.PBCR4 modified
*                                         PORT.PBCR3 modified
*                                         JCU.JCSTS deleted
*                                         VDC4.INP_SEL_CNT.INP_VSP_SYNC_SEL deleted
*                                         VDC4.SCL0_DS4.RES_DS_H_INIPHASE deleted
*                                         VDC4.GR1_AB1.GR1_ARC_ON deleted
*                                         VDC4.GR1_AB1.GR1_ARC_DISP_ON deleted
*                                         VDC4.GR1_AB4 deleted
*                                         VDC4.GR1_AB5 deleted
*                                         VDC4.GR1_AB6 deleted
*                                         VDC4.GR1_AB7.GR1_ARC_DEF deleted
*                                         VDC4.GR1_MON deleted
*                                         VDC4.ADJ_MTX_MODE.MTX_MD->ADJ_MTX_MD changed
*                                         VDC4.OUT_SET.OUT_PIXEL_INV_ON deleted
*                                         VDC4.OUT_SET.OUT_SUM_MOVE deleted
*               : Mar.02.2011 Ver.0.09.00 JCU.JCQTBL0 modified
*                                         JCU.JCQTBL1 modified
*                                         JCU.JCQTBL2 modified
*                                         JCU.JCQTBL3 modified
*                                         JCU.JCHTBD0 modified
*                                         JCU.JCHTBA0 modified
*                                         JCU.JCHTBD1 modified
*                                         JCU.JCHTBA1 modified
*               : Apr.04.2011 Ver.0.10.00 CPG.SWRSTCR2.JCUSRST added
*               : May.09.2011 Ver.0.11.00 BSC.ACSWR deleted
*                                         BSC.ACKEYR deleted
*                                         USB.USBACSWR1 deleted
*******************************************************************************/

#ifndef _IODEFINE_H_
#define _IODEFINE_H_

#include "r_typedefs.h"
#include "typedefine.h"

enum iodefine_byte_select_t {
    L = 0, H = 1,
    LL= 0, LH = 1, HL = 2, HH = 3,
};

union iodefine_reg32_t
{
    uint32_t  UINT32;                                  /*  32-bit Access   */
    uint16_t  UINT16[2];                               /*  16-bit Access   */
    uint8_t   UINT8[4];                                /*  8-bit Access    */
};

union iodefine_reg32_16_t
{
    uint32_t  UINT32;                                  /*  32-bit Access   */
    uint16_t  UINT16[2];                               /*  16-bit Access   */
};

/* ==== includes each iodefine ==== */
#include "adc_iodefine.h"            /* for adc module */
#include "ceu_iodefine.h"            /* for CEU module */
#include "scif_iodefine.h"           /* for SCIF module */
#include "pfc_iodefine.h"            /* for PFC module */
#include "bsc_iodefine.h"            /* for BSC module */
#include "cpg_iodefine.h"            /* for CPG module */
#include "dmac_iodefine.h"           /* for DMAC module */
#include "dvdec_iodefine.h"          /* for DVDEC module */
#include "gpio_iodefine.h"           /* for GPIO module */
#include "intc_iodefine.h"           /* for INTC module */
#include "ostm_iodefine.h"           /* for OSTM module */
#include "riic_iodefine.h"           /* for RIIC module */
#include "rspi_iodefine.h"           /* for RSPI module */
#include "prr_iodefine.h"            /* for Unprintable character */
#include "spibsc_iodefine.h"         /* for SPIBSC module */
#include "mtu2_iodefine.h"           /* for MTU module */
#include "jcu_iodefine.h"            /* for JCU module */
#include "pfv_iodefine.h"            /* for PFV module */
#include "lvds_iodefine.h"           /* for LVDS module */
#include "vdc5_iodefine.h"           /* for VDC module */
#include "wdt_iodefine.h"            /* for WDT module */

#endif /* _IODEFINE_H_ */
