/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name     : ramTest.h
* Device(s)     : RZ/A1H (R7S721001)
* Tool-Chain    : GNUARM-RZv14.01-EABI
* H/W Platform  : RSK+RZA1H CPU Board
* Description   : RAM tests module .h file
*******************************************************************************/
/*******************************************************************************
* History       : DD.MM.YYYY Version Description
*               : 21.08.2015 1.00    Initial Version
*******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

#ifndef RAMTEST_H_INCLUDED

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include <stdlib.h>
#include "r_typedefs.h"

#define RAMTEST_H_INCLUDED

/******************************************************************************
Macro definitions
******************************************************************************/

#define SDRAM_START ((bus_width_t*)0x08000000)
#define SDRAM_END    ((bus_width_t*)0x0A000000)
#define SDRAM_SIZE     ((SDRAM_END - SDRAM_START) / sizeof(bus_width_t))

#define MEM_DELAY_CNT               (10)

/******************************************************************************
Typedef definitions
******************************************************************************/

/* Define the size of the external bus for the SDRAM here */
typedef uint32_t bus_width_t;

/******************************************************************************
Function Prototypes
******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
Function Name: ramSizeTest
Description:   Function to perform a simple non-destructive RAM size test 
Arguments:     IN  pbwRamStart - Pointer to the start of the RAM
               IN  stLength - The length of the RAM in BUS_WIDTH
Return value:  The length of the RAM which passed the test or 0 on failure
******************************************************************************/

extern  size_t ramSizeTest(bus_width_t *pbwRamStart, size_t stLength);

/******************************************************************************
Function Name: ramFillTest
Description:   Function to fill the RAM with test data and read it back
Arguments:     IN  pbwRamStart - Pointer to the start of the RAM
               IN  stLength - The length of the RAM in bytes
Return value:  Pointer to the first location that fails the read back verify or
               0 on success
******************************************************************************/

extern  bus_width_t *ramFillTest(bus_width_t *pbwRamStart, size_t stLength);

#ifdef __cplusplus
}
#endif

#endif /* RAMTEST_H_INCLUDED */

/******************************************************************************
End  Of File
******************************************************************************/
