/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name     : intc.c
* Device(s)     : RZ/A1H (R7S721001)
* Tool-Chain    : GNUARM-RZv14.01-EABI
* H/W Platform  : RSK+RZA1H CPU Board
* Description   : interrupt handlier file
*******************************************************************************/
/*******************************************************************************
* History       : DD.MM.YYYY Version Description
*               : 21.08.2015 2.00
*******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "r_typedefs.h"
#include "dev_drv.h"                /* Device Driver common header */
#include "devdrv_intc.h"            /* INTC Driver Header */
#include "iodefine.h"
#include "compiler_settings.h"

/******************************************************************************
Macro definitions
******************************************************************************/
/* ==== Total number of registers ==== */
#define INTC_ICDISR_REG_TOTAL   (((uint16_t)INTC_ID_TOTAL / 32) + 1) /* ICDISR  */
#define INTC_ICDICFR_REG_TOTAL  (((uint16_t)INTC_ID_TOTAL / 16) + 1) /* ICDICFR */
#define INTC_ICDIPR_REG_TOTAL   (((uint16_t)INTC_ID_TOTAL /  4) + 1) /* ICDIPR  */
#define INTC_ICDIPTR_REG_TOTAL  (((uint16_t)INTC_ID_TOTAL /  4) + 1) /* ICDIPTR */
#define INTC_ICDISER_REG_TOTAL  (((uint16_t)INTC_ID_TOTAL / 32) + 1) /* ICDISER */
#define INTC_ICDICER_REG_TOTAL  (((uint16_t)INTC_ID_TOTAL / 32) + 1) /* ICDICER */

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
/* ==== Global variable ==== */
/* Initial value table of Interrupt Configuration Registers */
static uint32_t intc_icdicfrn_table[] =     
{                          /*           Interrupt ID */
    0xAAAAAAAA,            /* ICDICFR0  :  15 to   0 */
    0x00000055,            /* ICDICFR1  :  19 to  16 */
    0xFFFD5555,            /* ICDICFR2  :  47 to  32 */
    0x555FFFFF,            /* ICDICFR3  :  63 to  48 */
    0x55555555,            /* ICDICFR4  :  79 to  64 */
    0x55555555,            /* ICDICFR5  :  95 to  80 */
    0x55555555,            /* ICDICFR6  : 111 to  96 */
    0x55555555,            /* ICDICFR7  : 127 to 112 */
    0x5555F555,            /* ICDICFR8  : 143 to 128 */
    0x55555555,            /* ICDICFR9  : 159 to 144 */
    0x55555555,            /* ICDICFR10 : 175 to 160 */
    0xF5555555,            /* ICDICFR11 : 191 to 176 */
    0xF555F555,            /* ICDICFR12 : 207 to 192 */
    0x5555F555,            /* ICDICFR13 : 223 to 208 */
    0x55555555,            /* ICDICFR14 : 239 to 224 */
    0x55555555,            /* ICDICFR15 : 255 to 240 */
    0x55555555,            /* ICDICFR16 : 271 to 256 */
    0xFD555555,            /* ICDICFR17 : 287 to 272 */
    0x55555557,            /* ICDICFR18 : 303 to 288 */
    0x55555555,            /* ICDICFR19 : 319 to 304 */
    0x55555555,            /* ICDICFR20 : 335 to 320 */
    0x5F555555,            /* ICDICFR21 : 351 to 336 */
    0xFD55555F,            /* ICDICFR22 : 367 to 352 */
    0x55555557,            /* ICDICFR23 : 383 to 368 */
    0x55555555,            /* ICDICFR24 : 399 to 384 */
    0x55555555,            /* ICDICFR25 : 415 to 400 */
    0x55555555,            /* ICDICFR26 : 431 to 416 */
    0x55555555,            /* ICDICFR27 : 447 to 432 */
    0x55555555,            /* ICDICFR28 : 463 to 448 */
    0x55555555,            /* ICDICFR29 : 479 to 464 */
    0x55555555,            /* ICDICFR30 : 495 to 480 */
    0x55555555,            /* ICDICFR31 : 511 to 496 */
    0x55555555,            /* ICDICFR32 : 527 to 512 */
    0x55555555,            /* ICDICFR33 : 543 to 528 */
    0x55555555,            /* ICDICFR34 : 559 to 544 */
    0x55555555,            /* ICDICFR35 : 575 to 560 */
    0x00155555             /* ICDICFR36 : 586 to 576 */
};

/******************************************************************************
* Function Name: R_INTC_RegistIntFunc
* Description  : Registers the function specified by the func to the element 
*              : specified by the int_id in the INTC interrupt handler function
*              : table.
* Arguments    : uint16_t int_id         : Interrupt ID
*              : void (* func)(uint32_t) : Function to be registered to INTC
*              :                         : interrupt hander table
* Return Value : DEVDRV_SUCCESS          : Success of registration of INTC 
*              :                         : interrupt handler function
*              : DEVDRV_ERROR            : Failure of registration of INTC 
*              :                         : interrupt handler function
******************************************************************************/
int32_t R_INTC_RegistIntFunc(uint16_t int_id, void (* func)(uint32_t))
{
    if (int_id >= INTC_ID_TOTAL)
    {
        return DEVDRV_ERROR;
    }

    Userdef_INTC_RegistIntFunc(int_id, func);

    return DEVDRV_SUCCESS;
}
/*****************************************************************************
End of function  R_INTC_RegistIntFunc
******************************************************************************/

/******************************************************************************
* Function Name: R_INTC_Init
* Description  : Executes initial setting for the INTC.
*              : The interrupt mask level is set to 31 to receive interrupts 
*              : with the interrupt priority level 0 to 30.
* Arguments    : none
* Return Value : none
******************************************************************************/
void R_INTC_Init(void)
{
    uint16_t offset;
    volatile uint32_t * paddr;

    /* Interrupt Security Registers setting */
    paddr = (volatile uint32_t *)&INTC.ICDISR.LONG;
    for (offset = 0; offset < INTC_ICDISR_REG_TOTAL; offset++)
    {
        (*(paddr + offset)) = 0xFFFFFFFFuL;    /* Set all interrupts to be unsecured */
    }

    /* Interrupt Configuration Registers setting */
    paddr = (volatile uint32_t *)&INTC.ICDICFR.LONG;
    for (offset = 0; offset < INTC_ICDICFR_REG_TOTAL; offset++)
    {
        (*(paddr + offset)) = intc_icdicfrn_table[offset];
    }

    paddr = (volatile uint32_t *)&INTC.ICDIPR0.LONG;
    for (offset = 0; offset < INTC_ICDIPR_REG_TOTAL; offset++)
    {
        (*(paddr + offset)) = 0xF8F8F8F8uL;
    }

    /* Interrupt Processor Targets Registers setting */
    /* Initialize ICDIPTR8 to ICDIPTRn                     */
    /* (n = The number of interrupt sources / 4)           */
    /*   - ICDIPTR0 to ICDIPTR4 are dedicated for main CPU */
    /*   - ICDIPTR5 is dedicated for sub CPU               */
    /*   - ICDIPTR6 to 7 are reserved                      */
    paddr = (volatile uint32_t *)&INTC.ICDIPTR0.LONG;
    for (offset = 8; offset < INTC_ICDIPTR_REG_TOTAL; offset++)     /* Do not initialize ICDIPTR0 to ICDIPTR7 */
    {
        (*(paddr + offset)) = 0x01010101uL;
    }

    /* Interrupt Clear-Enable Registers setting */
    paddr = (volatile uint32_t *)&INTC.ICDICER.LONG;
    for (offset = 0; offset < INTC_ICDICER_REG_TOTAL; offset++)
    {
        (*(paddr + offset)) = 0xFFFFFFFFuL;
    }

    /* Interrupt Security Registers setting */
    paddr = (volatile uint32_t *)&INTC.ICDISR.LONG;
    for (offset = 0; offset < INTC_ICDISR_REG_TOTAL; offset++)
    {
        (*(paddr + offset)) = 0x00000000uL;    /* Set all interrupts to be secured */
    }


    /* ==== Initial setting for CPU interface ==== */
    /* Interrupt Priority Mask Register setting */
    R_INTC_SetMaskLevel(31);

    /* Binary Point Register setting */
    INTC.ICCBPR.LONG = 0x00000002uL;             /* Group priority field [7:3], Subpriority field [2:0](Do not use) */

    /* CPU Interface Control Register setting */
    INTC.ICCICR.LONG = 0x00000003uL;

    /* ==== Initial setting 2 to receive GIC interrupt request ==== */
    /* Distributor Control Register setting */
    INTC.ICDDCR.LONG = 0x00000001uL;
}
/*****************************************************************************
End of function  R_INTC_Init
******************************************************************************/

/******************************************************************************
* Function Name: R_INTC_Enable
* Description  : Enables interrupt of the ID specified by the int_id.
* Arguments    : uint16_t int_id : Interrupt ID
* Return Value : DEVDRV_SUCCESS  : Success to enable INTC interrupt
*              : DEVDRV_ERROR    : Failure to enable INTC interrupt
******************************************************************************/
int32_t R_INTC_Enable(uint16_t int_id)
{
    uint32_t reg_value;
    uint32_t mask;

    if (int_id >= INTC_ID_TOTAL)
    {
        return DEVDRV_ERROR;
    }

    /* ICDISERn has 32 sources in the 32 bits               */
    /* The n can be calculated by int_id / 32               */
    /* The bit field width is 1 bit                         */
    /* The target bit can be calclated by (int_id % 32) * 1 */
    /* ICDICERn does not effect on writing "0"              */
    /* The bits except for the target write "0"             */
    mask = 1;
    mask = mask << (int_id % 32);

    reg_value = INTC.ICDISER.LONG[int_id / 32];
    reg_value |= mask;
    INTC.ICDISER.LONG[int_id / 32] = reg_value;

    return DEVDRV_SUCCESS;
}
/*****************************************************************************
End of function  R_INTC_Enable
******************************************************************************/

/******************************************************************************
* Function Name: R_INTC_Disable
* Description  : Disables interrupt of the ID specified by the int_id.
* Arguments    : uint16_t int_id : Interrupt ID
* Return Value : DEVDRV_SUCCESS  : Success to disable INTC interrupt
*              : DEVDRV_ERROR    : Failure to disable INTC interrupt
******************************************************************************/
int32_t R_INTC_Disable(uint16_t int_id)
{
    uint32_t reg_value;
    uint32_t mask;

    if (int_id >= INTC_ID_TOTAL)
    {
        return DEVDRV_ERROR;
    }

    /* ICDICERn has 32 sources in the 32 bits               */
    /* The n can be calculated by int_id / 32               */
    /* The bit field width is 1 bit                         */
    /* The targe bit can be calculated by (int_id % 32) * 1 */
    /* ICDICERn does no effect on writing "0"               */
    /* Other bits except for the target write "0"           */
    mask = 1;
    mask = mask << (int_id % 32);

    reg_value = INTC.ICDICER.LONG[int_id / 32];
    reg_value |= mask;
    INTC.ICDICER.LONG[int_id / 32] = reg_value;

    return DEVDRV_SUCCESS;
}
/*****************************************************************************
End of function  R_INTC_Disable
******************************************************************************/

/******************************************************************************
* Function Name: R_INTC_SetPriority
* Description  : Sets the priority level of the ID specified by the int_id to 
*              : the priority level specified by the priority.
* Arguments    : uint16_t int_id   : Interrupt ID
*              : uint8_t  priority : Interrupt priority level (0 to 31)
* Return Value : DEVDRV_SUCCESS    : Success of INTC interrupt priority level setting
*              : DEVDRV_ERROR      : Failure of INTC interrupt priority level setting
******************************************************************************/
int32_t R_INTC_SetPriority(uint16_t int_id, uint8_t priority)
{
    uint32_t icdipr;
    uint32_t mask;
    volatile uint32_t * paddr;

    if (int_id >= INTC_ID_TOTAL)
    {
        return DEVDRV_ERROR;
    }

    priority = priority << 3;

    /* ICDIPRn has 4 sources in the 32 bits                 */
    /* The n can be calculated by int_id / 4                */
    /* The bit field width is 8 bits                        */
    /* The target bit can be calculated by (int_id % 4) * 8 */
    paddr = (volatile uint32_t *)&INTC.ICDIPR0.LONG;

    icdipr = *(paddr + (int_id / 4));

    mask = (uint32_t)0x000000FF;
    mask = mask << ((int_id % 4) * 8);
    icdipr &= ~mask;
    mask = (uint32_t)priority;
    mask = mask << ((int_id % 4) * 8);
    icdipr |= mask;

    *(paddr + (int_id / 4)) = icdipr;

    return DEVDRV_SUCCESS;
}
/*****************************************************************************
End of function  R_INTC_SetPriority
******************************************************************************/

/******************************************************************************
* Function Name: R_INTC_SetMaskLevel
* Description  : Sets the interrupt mask level specified by the mask_level.
* Arguments    : uint8_t mask_level : Interrupt mask level (0 to 31)
* Return Value : DEVDRV_SUCCESS     : Success of INTC interrupt mask level setting
*              : DEVDRV_ERROR       : Failure of INTC interrupt mask level setting
******************************************************************************/
void R_INTC_SetMaskLevel(uint8_t mask_level)
{
    mask_level = mask_level << 3;
    INTC.ICCPMR.BIT.Priority = mask_level;
}
/*****************************************************************************
End of function  R_INTC_SetMaskLevel
******************************************************************************/

/******************************************************************************
* Function Name: R_INTC_GetMaskLevel
* Description  : Obtains the setting value of the interrupt mask level, and 
*              : returns the obtained value to the mask_level.
* Arguments    : uint8_t * mask_level : Interrupt mask level (0 to 31)
* Return Value : none
******************************************************************************/
void R_INTC_GetMaskLevel(uint8_t * mask_level)
{
    *mask_level = INTC.ICCPMR.BIT.Priority;
    *mask_level = *mask_level >> 3;
}
/*****************************************************************************
End of function  R_INTC_GetMaskLevel
******************************************************************************/

/* END of File */

