/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name     : r_bsc.c
* Device(s)     : RZ/A1H (R7S721001)
* Tool-Chain    : GNUARM-RZv14.01-EABI
* H/W Platform  : RSK+RZA1H CPU Board
* Description   : Bus State Controller Initialisation Module
*******************************************************************************/
/*******************************************************************************
* History       : DD.MM.YYYY Version Description
*               : 21.08.2015 1.00    Initial Version
*******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "r_typedefs.h"
#include "dev_drv.h"             /* Device Driver common header */
#include "devdrv_common.h"       /* Common Driver Header */
#include "r_bsc.h"
#include "compiler_settings.h"

/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/******************************************************************************
* Function Name: R_BSC_Init
* Description  :
* Arguments    : uint8 area
*              :   B'xxxxxxxx
*              :     |||||||+--- [0]   CS0
*              :     ||||||+---- [1]   CS1
*              :     |||||+----- [2]   CS2
*              :     ||||+------ [3]   CS3
*              :     |||+------- [4]   CS4
*              :     ||+-------- [5]   CS5
*              :     ++--------- [6-7] n/a
* Return Value : none
******************************************************************************/
void RESET_CODE_SECTION R_BSC_Init(uint8_t area)
{
    /* ==== BSC initialize ==== */
    if ((area & BSC_AREA_CS0) != 0)     /* CS0 */
    {
        R_BSC_UserdefCs0Init();
    }
    if ((area & BSC_AREA_CS1) != 0)     /* CS1 */
    {
        R_BSC_UserdefCs1Init();
    }
    if ((area & BSC_AREA_CS2) != 0)     /* CS2 */
    {
        R_BSC_UserdefCs2Init();
    }
    if ((area & BSC_AREA_CS3) != 0)     /* CS3 */
    {
        R_BSC_UserdefCs3Init();
    }
    if ((area & BSC_AREA_CS4) != 0)     /* CS4 */
    {
        R_BSC_UserdefCs4Init();
    }
    if ((area & BSC_AREA_CS5) != 0)     /* CS5 */
    {
        R_BSC_UserdefCs5Init();
    }
}
/******************************************************************************
* End of Function R_BSC_Init
******************************************************************************/

/* End of File */

