/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name     : r_pmod_sample_main.c
* Device(s)     : RZ/A1H (R7S721001)
* Tool-Chain    : GNUARM-RZv14.01-EABI
* H/W Platform  : RSK+RZA1H CPU Board
* Description   : PMOD display board sample 
*******************************************************************************/
/*******************************************************************************
* History       : DD.MM.YYYY Version Description
*               : 21.08.2015 1.00    Initial Version
*******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
/* Standard includes. */
#include <stdlib.h>
#include <stdio.h>

#include "r_typedefs.h"
#include "r_task.h"
#include "iodefine.h"

/* Ensure local malloc is used */
#include "..\inc\malloc.h"

#include "command.h"
#include "console.h"
#include "trace.h"
#include "hwRIIC3.h"
#include "hwLCDPMod.h"


/* GNU Compiler settings */
#include "compiler_settings.h"
#include "r_pmod_samplemain.h"
#include "r_lcd_pmod.h"
#include "main.h"
#include "r_src_resource.h"

/* Comment this line out to turn ON module trace in this file */
/* #undef _TRACE_ON_ */

#ifndef _TRACE_ON_
#undef TRACE
#define TRACE(x)
#endif

extern unsigned char g_acDesert[];
extern unsigned char g_acHydrangeas[];
extern unsigned char g_acPenguins[];

/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/
/* Define the command tables required */
extern PCMDTAB   gppCommands[];
extern int32_t   giNumCommands;

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/
static int cmd_pmod_start (int iArgCount, char **ppszArgument, PCOMSET pCom);
static int cmd_pmod_rotate (int iArgCount, char **ppszArgument, PCOMSET pCom);
static int cmd_pmod_stop (int iArgCount, char **ppszArgument, PCOMSET pCom);
static void task_execute_pmod(void);

/* Table that associates command letters, function pointer and a little
 description of what the command does */
CMDFNASS gpcmdPMODSample[] =
{
    {
        "pmodstart",
        cmd_pmod_start,
        "<CR> - Run PMOD sample"
    },

    {
        "pmodstop",
        cmd_pmod_stop,
        "<CR> - Terminate PMOD sample"
    },

    {
        "pmodrotate",
        cmd_pmod_rotate,
        "<CR> - Rotate image on PMOD display by 90 degrees clockwise.\n"
    }
};

/* Table that points to the above table and contains the number of entries */
static CMDTAB gcmdPMODSample =
{
    gpcmdPMODSample,
    ((sizeof(gpcmdPMODSample))/(sizeof(CMDFNASS)))
};

/******************************************************************************
Private global variables and functions
******************************************************************************/

/* Create the console to run the commands */
COMSET * pcmd_set;

static ppmod_config_t g_pmod_pactive_if = NULL;


/******************************************************************************
* Function Name: R_PMOD_SampleMain
* Description  :
* Arguments    : none
* Return Value : none
******************************************************************************/
void R_PMOD_SampleMain(FILE *pIn, FILE *pOut)
{
    uint32_t ui_events;

    if (0 == g_pmod_pactive_if)
    {
        g_pmod_pactive_if= pvPortMalloc(sizeof(pmod_config_t));

        if (0 != g_pmod_pactive_if)
        {
            memset(g_pmod_pactive_if, 0, sizeof(pmod_config_t));

            g_pmod_pactive_if->bfActive = false;
            g_pmod_pactive_if->image_angle = 1;

            /* Create the signals */
            ui_events = eventCreate(g_pmod_pactive_if->ppevSignals, PMOD_NUM_SIGNALS);
        }

        if (ui_events)
        {
            eventDestroy(g_pmod_pactive_if->ppevSignals, PMOD_NUM_SIGNALS-ui_events);
            printf("R_PMOD_SampleMain ui_events create failed\r\n");
        }
        else
        {
            if(false == eventSet(g_pmod_pactive_if->ppevSignals[PMOD_SAMPLE_CREATED]))
            {
                printf("R_PMOD_SampleMain PMOD_SAMPLE_CREATED failed\r\n");
            }
            else
            {
                int32_t pos;
                pcmd_set = malloc(sizeof(COMSET));

                /* create task */
                for(pos = 0; pos < giNumCommands; pos++)
                {
                    if(NULL == gppCommands[pos])
                    {
                        gppCommands[pos] = &gcmdPMODSample;
                        pos = giNumCommands;
                    }
                }
            }
        }
    }
}
/*******************************************************************************
End of function R_PMOD_SampleMain
*******************************************************************************/

/******************************************************************************
* Function Name: cmd_pmod_start
* Description  : 
* Arguments    : none
* Return Value : none
******************************************************************************/
static int cmd_pmod_start (int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    if (pcmd_set)
    {
        if(false == g_pmod_pactive_if->bfActive)
        {
            if(true == R_SCR_ResourceLockDisplay())
            {
                g_pmod_pactive_if->bfActive = true;

                g_pmod_pactive_if->taskid =
                               taskCreate((PTASKFN)task_execute_pmod,
                               NULL,
                               TC_SMALL_STACK_SIZE,
                               TC_DEFAULT_HEAP,
                               "PMOD Demo",
                               0);
            }
        }
        else
        {
            printf("cmd_pmod_start task running\r\n");
        }
    }
    return 0;
}
/******************************************************************************
* End of function cmd_pmod_start
******************************************************************************/

/******************************************************************************
* Function Name: cmd_pmod_stop
* Description  : 
* Arguments    : none
* Return Value : none
******************************************************************************/
static int cmd_pmod_stop (int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    if (pcmd_set)
    {
        if(true == g_pmod_pactive_if->bfActive)
        {
            g_pmod_pactive_if->bfActive = false;
            R_SCR_ResourceUnlockDisplay();
            lcd_pmod_init();
        }
        else
        {
            printf("cmd_pmod_start task not running\r\n");
        }
    }
    return 0;
}
/******************************************************************************
* End of function cmd_pmod_stop
******************************************************************************/

/******************************************************************************
* Function Name: cmd_pmod_rotate
* Description  :
* Arguments    : none
* Return Value : none
******************************************************************************/
static int cmd_pmod_rotate (int iArgCount, char **ppszArgument, PCOMSET pCom)
{
    if (pcmd_set)
    {
        R_LCD_ImageRotation(g_pmod_pactive_if->image_angle++);
        g_pmod_pactive_if->image_angle %= 4;
        if(0 == R_SCR_ResourceDisplayStatus())
        {
            R_LCD_UpdateDisplay();
        }
    }
    return 0;
}
/******************************************************************************
* End of function cmd_pmod_rotate
******************************************************************************/

/******************************************************************************
* Function Name: task_execute_pmod
* Description  : Test task to flash the LEDs
* Arguments    : none
* Return Value : none
******************************************************************************/
static void task_execute_pmod(void)
{
    int32_t       i_led_map = 0;
    const int32_t i_sleep_count = 1000UL;

        while(true == g_pmod_pactive_if->bfActive)
        {
            R_LCD_ResourceLockPMOD();
            switch(i_led_map)
            {
                case 0:
                {
                    R_LCD_DisplayBMPImage (&g_acDesert[0], 128, 128, 0, 0);
                    i_led_map++;
                }
                break;
                case 1:
                {
                    R_LCD_DisplayBMPImage (&g_acHydrangeas[0], 128, 128, 0, 0);
                    i_led_map++;
                }
                break;
                case 2:
                {
                    R_LCD_DisplayBMPImage (&g_acPenguins[0], 128, 128, 0, 0);
                    i_led_map = 0;
                }
                break;
                default:
                {
                    i_led_map = -1;
                }
            }
            taskSleep(i_sleep_count);
            R_LCD_UpdateDisplay();
            R_LCD_ResourceUnlockPMOD();
        }
    taskExit();
}
/******************************************************************************
End of function  task_monitor_sd
******************************************************************************/

/******************************************************************************
* Function Name: cmd_pmod_rotate
* Description  :
* Arguments    : none
* Return Value : none
******************************************************************************/
void pmod_rotate_no_screen_update (void)
{
    g_pmod_pactive_if->image_angle++;
    g_pmod_pactive_if->image_angle %= 4;
}
/*******************************************************************************
End of function pmod_rotate_no_screen_update
*******************************************************************************/

/* End of File */

