/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name     : r_sys_hardwaresetup.c
* Device(s)     : RZ/A1H (R7S721001)
* Tool-Chain    : GNUARM-RZv14.01-EABI
* H/W Platform  : RSK+RZA1H CPU Board
* Description   : Low Level  Hardware setup module user file
*******************************************************************************/
/*******************************************************************************
* History       : DD.MM.YYYY Version Description
*               : 21.08.2015 1.00    Initial Version
*******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "r_typedefs.h"
#include "devdrv_common.h"      /* Common Driver Header */
#include "iodefine.h"
#include "stb_init.h"
#include "port_init.h"

/******************************************************************************
Defines
******************************************************************************/

/******************************************************************************
Constant Macros
******************************************************************************/

/******************************************************************************
Function Macros
******************************************************************************/

/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Enumerated Types
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Constant Data
******************************************************************************/

/******************************************************************************
Private Function Prototypes
******************************************************************************/

static void initialise_leds(void);

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
* Function Name: R_SYS_HardwareSetup
* Description  : Function to initialise the hardware
* Arguments    : none
* Return Value : none
******************************************************************************/
void R_SYS_HardwareSetup(void)
{
    STB_Init();

    PORT_Init();

    /* ==== BSC setting ==== */
    /* Note BSC_AREA_CS0 & BSC_AREA_CS1 initialised by Peripheral_BasicInit() */
    /* R_BSC_Init((uint8_t)(BSC_AREA_CS2 | BSC_AREA_CS3)); */
    /* DO NOT ENABLE CS3 - This is part of SDRAM workaround. */
    R_BSC_Init((uint8_t)(BSC_AREA_CS2));

    initialise_leds();
}
/******************************************************************************
End of function  R_SYS_HardwareSetup
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/******************************************************************************
* Function Name: initialise_leds
* Description  : Function to initialise the port pins used to control the LEDs
* Arguments    : none
* Return Value : none
******************************************************************************/
static void initialise_leds(void)
{
    /* Initialise P7.1 for LED0 */
    PORT7.PMCn.BIT.PMCn1 = 0;
    PORT7.Pn.BIT.Pn1 = 1;
    PORT7.PMn.BIT.PMn1 = 0;
    PORT7.PIPCn.BIT.PIPCn1 = 0;
}
/******************************************************************************
End of function  initialise_leds
******************************************************************************/

/******************************************************************************
End  Of File
*******************************************************************************/
