/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : main.c
* Version      : 1.02
* Device(s)    : Renesas
* Tool-Chain   : Renesas SH2A V9+
* OS           : FreeRTOS
* H/W Platform : RSK2+RZA1
* Description  : Web Engine Sample Program
*******************************************************************************
* History      : DD.MM.YYYY Version Description

*              : 04.02.2010 1.00    First Release
*              : 10.06.2010 1.01    Updated type definitions
*              : 31.03.2011 1.02    Converted to Object Oriented Ethernet
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include <stdio.h>
#include <string.h>


#include "FreeRTOS.h"
#include "r_task.h"
#include "command.h"
#include "control.h"
#include "taskPriority.h"
#include "sysUsage.h"
#include "version.h"
#include "trace.h"
#include "hwLCDPMod.h"
#include "compiler_settings.h"
#include "main.h"

#include "r_pmod_samplemain.h"

/* START INSERT SAMPLE HEADER FILE INCLUDES HERE */

/* END INSERT SAMPLE HEADER FILE INCLUDES HERE */

/******************************************************************************
Typedef definitions
******************************************************************************/

typedef struct _iostr_t
{
    FILE    *pIn;
    FILE    *pOut;
} iostr_t;

typedef iostr_t *p_iostr_t;


/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Function Prototypes
******************************************************************************/
static void command_shell(p_iostr_t p_iostr_t);

/******************************************************************************
Constant Data
******************************************************************************/

/******************************************************************************
Public Functions
******************************************************************************/

/******************************************************************************
Function Name: main
Description:   Main entry point of the application
Arguments:     none
Return value:  none
******************************************************************************/
int main(void)
{
	iostr_t p_iostr[3];

    /* Assign the streams for the serial console */
    p_iostr[0].pIn = stdin;
    p_iostr[0].pOut = stdout;

    fprintf(p_iostr->pOut,"\r\n");
    fflush(p_iostr->pOut);

    r_RIIC3_Initialise();

    /* Initialise PMOD */
    lcd_pmod_init();
    R_PMOD_SampleMain(p_iostr->pIn,p_iostr->pOut);

    /* START INSERT SAMPLE MAIN FUNCTION CALLS HERE */

    /* END INSERT SAMPLE MAIN FUNCTION CALLS HERE */

    /* Do the serial console */
    command_shell(p_iostr);

    /* Close the streams */
    fclose(p_iostr[2].pIn);

    /* Exit from main returns to rstprg.c where low power mode is entered */
}
/******************************************************************************
End of function  main
******************************************************************************/

/******************************************************************************
Function Name: abort
Description:   Function called by the ASSERT macro
Parameters:    none
Return value:  none
******************************************************************************/
void abort(void)
{
    fprintf(stderr, "Press any key to continue\r\n");
    fgetc(stdin);
    while(1)
    {
    	/* ensure function does not return */
    };
}
/******************************************************************************
End of function  abort
******************************************************************************/

/******************************************************************************
Private functions
******************************************************************************/

/*****************************************************************************
* Function Name: command_shell
* Description:   Task to perform a control console
* Arguments:     IN  p_iostr_t - Pointer to the IO streams to use
* Return value:  none
*****************************************************************************/
static void command_shell(p_iostr_t p_iostr)
{
    /* Do the console function */
    cmdConsole(p_iostr->pIn, p_iostr->pOut, CMD_LINE_PROMPT);
}
/******************************************************************************
End of function  command_shell
******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
